import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Aesthetic & Anti-Aging Physician related titles (Variants)
aesthetic_antiaging_physician_variants = [
    r'\b(?:Aesthetic & Anti Aging|Aesthetic & AntiAging|Aesthetic & Antiaging)\b',
    r"(?i)\bAesthetic Antiage & Resgenerative Medicine\b",
    r"(?i)\bAppearance Medicine\b",
    r"(?i)\bSkin Rejuvenation & Resurfacing\b",
    r"(?i)\bNon Surgery\b",
    r"(?i)\bAesthetic Body Contouring Specialist\b",
    r"(?i)\bAesthetic&Anti-Aging Physcian\b",
    r"(?i)\bDr  Aesthetic & Anti-Aging Medicine\b",
]

# Exact matches that should be updated
aesthetic_antiaging_physician_exact_matches = {
    'Aesthetic & Anti Aging Physician',
    'Aesthetic & AntiAging Physician',
    'Aesthetic & Antiaging Physician',
    'Aesthetic & Anti Aging Practitioner',
    'Aesthetic & AntiAging Practitioner',
    'Aesthetic & Antiaging Practitioner',
    'Aesthetic & Anti-Aging Practitioner',
    'Aesthetic Anti-Aging Physician',
    'Aesthetic Anti Aging Physician',
    'Aesthetic AntiAging Physician',
    'Aesthetic Antiaging Physician',
    'Aesthetic Anti Aging Medical Doctor',
    'Aesthetic AntiAging Medical Doctor',
    'Aesthetic Antiaging Medical Doctor',
    'Aesthetic Anti Aging Specialist',
    'Aesthetic AntiAging Specialist',
    'Aesthetic Antiaging Specialist',
    'Aesthetic Anti Aging Doctor',
    'Aesthetic AntiAging Doctor',
    'Aesthetic Antiaging Doctor',
    'Aesthetic Anti Aging Surgeon',
    'Aesthetic AntiAging Surgeon',
    'Aesthetic Antiaging Surgeon',
    'Anti-Aging Aesthetic Physician',
    'Anti-Aging Aesthetic Practitioner',
    'Anti Aging Aesthetic Physician',
    'Anti Aging Aesthetic Practitioner',
    'Aesthetic & Anti Aging Medical Practitioner',
    'Aesthetic & AntiAging Medical Practitioner',
    'Aesthetic & Antiaging Medical Practitioner',
    'Aesthetic & Anti Aging Healthcare Practitioner',
    'Aesthetic & AntiAging Healthcare Practitioner',
    'Aesthetic & Antiaging Healthcare Practitioner',
    'Aesthetic Anti Aging Aesthetician',
    'Aesthetic & Anti Aging Medical Specialist',
    'Aesthetic & AntiAging Medical Specialist',
    'Aesthetic & Antiaging Medical Specialist',
    'Aesthetic & Anti-Aging Physician',
# Variations for common typos and case mistakes
    'Aesthetic & Anti-Aging Physician',
    'Aesthetic & AntiAging Physiican',
    'Aesthetic Anti-Aging Physican',
    'Aesthetic Anti Aging Physcian',
    'Aesthetic Anti Aging Pracitioner',
    'Aesthetic Anti-Aging Medical Doctor',
    'Aesthetic AntiAging Medical Dotor',
    'Aesthetic Anti Aging Speciallist',
    'Aesthetic Anti Aging Pracitioner',
    'Aesthetic Anti Aging Practicioner',
    'Aesthetic AntiAging Pracitioner',
    'Aesthetic Antiaging Medical Doctor',
    'Aesthetic AntiAging Surgeion',
    'Aesthetic Anti Aging Medcial Specalist',
    'Aesthetic Anti Aging Physican',
    'Aesthetic Anti-Aging Speciallist',
    'Aesthetic Anti Aging Healthcare Practicioner',
    'Aesthetic AntiAging Specilist',
    'Aesthetic Anti Aging Aesthetician',
    'Aesthetic AntiAging Surgeion',
    'Aesthetic Antiaging Doctor',
    'Aesthetic Antiaging Healthcare Specialist',
    'Aesthetic & Anti-Aging Practicioner',

    # values from data sources
    'Mesotherapist',
    'Mesapist',
    'Injectables',
    'Anti-Aging/Aesthetic Specialty',
    'Anti-AgingAesthetic Specialty',
    'Botox',
    'Body Contouring',
    'Aesthetic & Anti-Aging',
    'Aesthetics & Wellness Director',
    'Aesthetic & Medical Wellness Doctor',
    'GpAesthetic & Longevity Doctor',
    'Family MedecineAesthetic Anti-Aging Medecine',
    'Aesthetic & Wellness',
    'Aesthetic & Wellness Physicians At Monaco Wellness Tampa Fl',
    'Body Contouring Specialist',
    'body contouring specialist',
    'Aesthetic & Anti-aging Physcian',
}

# Define patterns for Anti-aging & Resident (these should NOT be changed)
aesthetic_antiaging_physician_exclusions = {
    'Anti-aging Physician',
    'Anti Aging Physician',
    'anti aging physician',
    'anti-aging physician',
    'Antiaging Physician',
    'AntiAging Physician',
    'antiaging physician',
    'antiAging physician',
    'anti-ageing physician',
    'anti ageeing physician',
    'Anti-Aging Dr.',
    'Anti Aging Dr.',
    'anti-aging dr',
    'anti aging dr',
    'Anti-Aging Specialist',
    'Anti Aging Specialist',
    'anti aging specialist',
    'anti-aging specialist',
    'Anti-aging Practitioner',
    'Anti Aging Practitioner',
    'anti-aging practitioner',
    'anti aging practitioner',
    'Anti-aging Physician Clinic',
    'Anti Aging Physician Clinic',
    'anti-aging physician clinic',
    'anti aging physician clinic',
    'Anti-aging MD',
    'Anti Aging MD',
    'anti-aging md',
    'anti aging md',
    'Anti-Aging Medical Doctor',
    'Anti Aging Medical Doctor',
    'anti-aging medical doctor',
    'anti aging medical doctor',
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'RN Injection & Body Contouring Specialist',
}

# Create a mask for aesthetics_antiaging_physician
mask_aesthetic_antiaging_physician = df['speciality'].str.contains('|'.join(aesthetic_antiaging_physician_variants), case=False, na=False, regex=True) | \
            df['speciality'].isin(aesthetic_antiaging_physician_exact_matches)
mask_aesthetic_antiaging_physician_exclusions = df['speciality'].isin(aesthetic_antiaging_physician_exclusions)

# Final mask: Select Aesthetic & Anti-aging Physician -related values
mask_aesthetic_antiaging_physician_final = mask_aesthetic_antiaging_physician & ~mask_aesthetic_antiaging_physician_exclusions

# Store the original values that will be replaced
original_aesthetics_antiaging_physician_values = df.loc[mask_aesthetic_antiaging_physician_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_aesthetic_antiaging_physician_final, 'speciality'] = 'Aesthetic & Anti-aging Physician'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Aesthetic & Anti-aging Physician", 'green'))
print(df.loc[mask_aesthetic_antiaging_physician_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_aesthetics_antiaging_physician_values = df.loc[mask_aesthetic_antiaging_physician_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Aesthetic & Anti-aging Physician", "cyan"))
for original_aesthetics_antiaging_physician_value in original_aesthetics_antiaging_physician_values:
    print(f"✅ {original_aesthetics_antiaging_physician_value} → Aesthetic & Anti-aging Physician")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Aesthetic & Anti-aging Physician:", 'red'))
print(grouped_aesthetics_antiaging_physician_values)

# Print summary
matched_count_aesthetics_antiaging_physician = mask_aesthetic_antiaging_physician_final.sum()

# Print results
print(colored(f"\nTotal values matched and changed (Stage 1) to Aesthetic & Anti-aging Physician: {matched_count_aesthetics_antiaging_physician}", 'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Gynecologist Obstetrician related titles

gynecologist_variants = [
    r"(?i)\bGynecologist\s?Obstetrician\b",

    # Common misspellings and case errors
    r"(?i)\bGynaecologist\s?Obstetrician\b",
    r"(?i)\bGynecologist\s?Obstetrician\b",
    r"(?i)\bGynecologis\s?Obstetrician\b",
    r"(?i)\bObstetrician\s?Gynecologist\b",
    r"(?i)\bGynecologist\b",
    r"(?i)\bGynecology\b",
    r"(?i)\bObstetrician\b",
    r"(?i)\bObgyn\b",
    r"(?i)\bGynaecologist\b",

    # Spanish variants
    r"(?i)\bGinecólogo\s?Obstetra\b",
    r"(?i)\bGinecóloga\s?Obstetra\b",
    r"(?i)\bMédico\s?Ginecólogo\s?Obstetra\b",

    # Other possible variations
    r"(?i)\bDoctor\s?Gynecologist\s?Obstetrician\b",
    r"(?i)\bObstetrics\s?Gynecology\s?Specialist\b",
    r"(?i)\bOB/GYN\b",
    r"(?i)\bOB-GYN\b",
    r"(?i)\bBoard-Certified\s?Gynecologist\s?Obstetrician\b",
    r"(?i)Gynecologyst",
    r"(?i)\bMedica Ginecologista\b",
    r"(?i)Ginecologista",
    r"(?i)\bMedico Ginecologista E Obstetra\b",
    r"(?i)Obstetra",
    r"(?i)Gynecologogist",
    r"(?i)Consultantobstetricsgynecology",
    r"(?i)Ob Gyn",
    r"(?i)\bHpv\b",
    r"(?i)Gynaecology",
    r"(?i)Gynecologie",
    r"(?i)Gynecolog",
    r"(?i)Gynecologue",
    r"(?i)\bObs\b",
    r"(?i)Ginecologia",
    r"(?i)\bGynecology Esthetic Medicine\b",
    r"(?i)\bObgy\b",
    r"(?i)\bObstetrics & Gynaecology\b",
    r"(?i)\bObs&Gyne\b",
]

# Exact matches that should be updated
gynecologist_exact_matches = {
    "Gynecologist Obstetrician",
    "Gynaecologist Obstetrician",
    "Gynecologis Obstetrician",
    "Obstetrician Gynecologist",
    "Ginecólogo Obstetra",
    "Ginecóloga Obstetra",
    "Médico Ginecólogo Obstetra",
    # Other possible variations
    "Doctor Gynecologist Obstetrician",
    "Obstetrics Gynecology Specialist",
    "OB/GYN",
    "OB-GYN",
    "Board-Certified Gynecologist Obstetrician",
    'Hpv',
    'HPV',
    'Ob Gyn',
    'GYN',
    'Gyn',
    'obgyn',
    'OB GYN',
    'Gynakologie',
    'GynObst',
    'Rural Obstetrics'
    'Jinekolog',
    'MacDecins GynacCologue Esthetique',
    'Ginecologo',
    'Ginecologa',
    'Obg Spescialist',
    'Gyneco-Oncology',
    'Gin obst',
}

# Define patterns (these should NOT be changed)
gynecologist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Obgyn Physician Recruiter',
}

# Create a mask for Gynecologist Obstetrician
mask_gynecologist = df['speciality'].str.contains('|'.join(gynecologist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(gynecologist_exact_matches)
mask_gynecologist_exclusions = df['speciality'].isin(gynecologist_exclusions)

# Final mask: Select Gynecologist Obstetrician
mask_gynecologist_final = mask_gynecologist & ~mask_gynecologist_exclusions

# Store the original values that will be replaced
original_gynecologist_values = df.loc[mask_gynecologist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_gynecologist_final, 'speciality'] = 'Gynecologist Obstetrician'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Gynecologist Obstetrician", 'green'))
print(df.loc[mask_gynecologist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_gynecologist_values = df.loc[mask_gynecologist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Gynecologist Obstetrician", "cyan"))
for original_gynecologist_value in original_gynecologist_values:
    print(f"✅ {original_gynecologist_value} → Gynecologist Obstetrician")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Gynecologist Obstetrician:", 'red'))
print(grouped_gynecologist_values)

# Print summary
matched_count_gynecologist = mask_gynecologist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Gynecologist Obstetrician: "
        f"{matched_count_gynecologist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Cosmetic Dermatologist related titles

cosmetic_dermatologist_variants = [
    # Standard title variations
    r"(?i)\bCosmetic\s?Dermatologist\b",
    r"(?i)\bDermatologist\s?Cosmetic\b",
    r"(?i)\bCosmetic\s?Skin\s?Specialist\b",
    r"(?i)\bDoctor\s?of\s?Cosmetic\s?Dermatology\b",
    r"(?i)\bCosmetic\s?Skin\s?Doctor\b",
    r"(?i)\bAesthetic\s?Dermatologist\b",
    r"(?i)\bSkin\s?Care\s?Specialist\b",

    # Common misspellings and case mistakes
    r"(?i)\bCosmetic\s?Dermotologist\b",
    r"(?i)\bCosmetick\s?Dermatologist\b",
    r"(?i)\bCosmotik\s?Dermatologist\b",
    r"(?i)\bKosmetic\s?Dermatologist\b",
    r"(?i)\bCosmetic\s?Dermetologist\b",

    # Variants in Spanish and other languages
    r"(?i)\bDermatólogo\s?Cosmético\b",
    r"(?i)\bMédico\s?Estético\b",
    r"(?i)\bEspecialista\s?en\s?Dermatología\s?Cosmética\b",

    # Other possible variations
    r"(?i)\bBoard\s?-?\s?Certified\s?Cosmetic\s?Dermatologist\b",
    r"(?i)\bGeneral\s?Cosmetic\s?Dermatologist\b",
    r"(?i)\bCosmeti Dermatology\b",
    r"(?i)\bLikar Dermatolog\b",
    r"(?i)\bCosmetic Dermatplpgy\b",
    r"(?i)\bConsultant DermatologistCosmetologist\b",
    r"(?i)\bCosmetic Dermatology\b",
    r"(?i)\bCosmetic Injecting\b",
    r"(?i)\bMedical Cosmetic\b",
    r"(?i)\bDermal Filler\b",
    r"(?i)\bKosmetische Medizin\b",
    r"(?i)\bCosmetic Injectables\b",
    r"(?i)\bMedica Procedimento Estetico\b",
    r"(?i)\bEstetica Facial\b",
    r"(?i)\bVrach Dermatokosmetolog\b",
    r"(?i)\bDermatology - Medical Devices\b",
    r"(?i)\bFacial Rejuvenation & Improvement Of Skin Quality With A Combination Of Injectable Treatments Of Hyaluronic Acid Calcium Hydroxyapatite & Botulinum Toxin Type A A Case Report\b",
    r"(?i)\bAnti Wrinkle Tx\b",
    r"(?i)\bDermocosmetics\b",
    r"(?i)\bDermatologistCosmetologist\b",
    r"(?i)\bDermato Cosmetolog\b",
    r"(?i)\bEstetica E Cosmetica\b",
    r"(?i)\bCosmiatra\b",
    r"(?i)\bCosmetic Efficacy\b",
    r"(?i)\bAnti-Wrinkle\b",
]

# Exact matches that should be updated
cosmetic_dermatologist_exact_matches = {
    "Cosmetic Dermatologist",
    "cosmetic dermatologist",
    "COSMETIC DERMATOLOGIST",
    "Cosmetic Dermotologist",
    "Cosmetick Dermatologist",
    "Cosmotik Dermatologist",
    "Kosmetic Dermatologist",
    "Cosmetic Dermetologist",
    "Cosmeti Dermatology",
    'Dermal Fillers',
    'dermal fillers',
    'dermal filler',
    'Dermal Filler',

    # Variants in Spanish
    "Dermatólogo Cosmético",
    "Médico Estético",
    "Especialista en Dermatología Cosmética",

    # Hybrid terms (English-Spanish combinations)
    "Cosmetic Dermatólogo",
    "Cosmetico Dermatologist",

    # Values from data sources
    "Cosmetic Dermatology",
    "cosmetic dermatology",
    "Aesthetic Dermatologist",
    'Cosmetic MedicineAesthetics',
    'Cosmetisch Arts',
    'DirectorSenior Dermal',
    'Dermal',
    'Vrach Kosmetolog Dermatolog',
    'Cosmetic Department',
    'Prp',
    'PRP',
    'prp',
    'Dermatology Therapeutic Specialist',
    'Senior Dermatology Therapeutic Specialist',
}

# Define patterns (these should NOT be changed)
cosmetic_dermatologist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Registered Nurse Cosmetic Injectables',
}

# Create a mask for cosmetic_surgeon
mask_cosmetic_dermatologist = df['speciality'].str.contains('|'.join(cosmetic_dermatologist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(cosmetic_dermatologist_exact_matches)
mask_cosmetic_dermatologist_exclusions = df['speciality'].isin(cosmetic_dermatologist_exclusions)

# Final mask: Select cosmetic_surgeon
mask_cosmetic_dermatologist_final = mask_cosmetic_dermatologist & ~mask_cosmetic_dermatologist_exclusions

# Store the original values that will be replaced
original_cosmetic_dermatologist_values = df.loc[mask_cosmetic_dermatologist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_cosmetic_dermatologist_final, 'speciality'] = 'Cosmetic Dermatologist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Cosmetic Dermatologist", 'green'))
print(df.loc[mask_cosmetic_dermatologist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_cosmetic_dermatologist_values = df.loc[mask_cosmetic_dermatologist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Cosmetic Dermatologist", "cyan"))
for original_cosmetic_dermatologist_value in original_cosmetic_dermatologist_values:
    print(f"✅ {original_cosmetic_dermatologist_value} → Cosmetic Dermatologist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Cosmetic Dermatologist:", 'red'))
print(grouped_cosmetic_dermatologist_values)

# Print summary
matched_count_cosmetic_dermatologist = mask_cosmetic_dermatologist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Cosmetic Dermatologist: "
        f"{matched_count_cosmetic_dermatologist}",
        'red'))


# Consmetic Surgeon

# Define regex patterns for Cosmetic Surgeon related titles

cosmetic_surgeon_variants = [
    # Standard title variations
    r"(?i)\bCosmetic\s?Surgeon\b",
    r"(?i)\bSurgeon\s?Cosmetic\b",
    r"(?i)\bCosmetic\s?Surgery\s?Specialist\b",
    r"(?i)\bSurgeon\s?for\s?Cosmetic\s?Procedures\b",
    r"(?i)\bDoctor\s?of\s?Cosmetic\s?Surgery\b",
    r"(?i)\bCosmetic Physician\b",
    r"(?i)\bCosmetic Surgery\b",
    r"(?i)\bCosmetic & Reconstruction Surgeon\b",
    r"(?i)\bCosmetic Physicians\b",

    # Common misspellings and case mistakes
    r"(?i)\bCosmetic\s?Surgion\b",
    r"(?i)\bCosmetick\s?Surgeon\b",
    r"(?i)\bCosmotik\s?Surgeon\b",
    r"(?i)\bKosmetic\s?Surgeon\b",
    r"(?i)\bCosmetic\s?Surjen\b",

    # Variants in Spanish and other languages
    r"(?i)\bCirujano\s?Cosmético\b",
    r"(?i)\bCirujano\s?Estético\b",
    r"(?i)\bMédico\s?Estético\b",

    # Other possible variations
    r"(?i)\bAesthetic Surgeon\b",
    r"(?i)\bBoard\s?-?\s?Certified\s?Cosmetic\s?Surgeon\b",
    r"(?i)\bCosmeti Surgery\b",
    r"(?i)\bGeneralCosmetic Surgeon\b",
    r"(?i)\bLikar Kosmetolog\b",
    r"(?i)\bCosmetic Physicion\b",
    r"(?i)\bDr  Kosmetolog\b",
    r"(?i)\bCoametic Surgeon\b",
]

# Exact matches that should be updated
cosmetic_surgeon_exact_matches = {
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",
    "Cosmetic Surgeon",

    # Common misspellings for Cosmetic Surgeon titles
    "Cosmetic Surgion",
    "Cosmetick Surgeon",
    "Cosmotik Surgeon",
    "Kosmetic Surgeon",
    "Cosmetic Surjen",
    "Cosmetik Surjen",
    "Kosmetic Surjen",
    "Cosmetic Surgeoon",

    # Case-related errors for Cosmetic Surgeon
    "cosmetic surgeon",
    "COSMETIC SURGEON",
    "Cosmetic Surgeon",
    "cosmetik surgeon",
    "kosmetic surgeon",
    "cosmetic surjen",
    "cosmetic surgeoon",

    # Variants of Cosmetic Surgeon in Spanish
    "Cirujano Cosmético",
    "Cirujano Estético",
    "Médico Estético",

    # Hybrid terms (Including English-Spanish combinations)
    "Cosmetic Cirujano",
    "Cosmetico Surgeon",

    # values from data sources
    'Cosmetic Doctor',
    'CIRUGIA COSMETICA',
}

# Define patterns (these should NOT be changed)
cosmetic_surgeon_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Plastic Reconstructive & Aesthetic Surgeon',
    'Cosmetic Surgery Fellow In Training',
}

# Create a mask for cosmetic_surgeon
mask_cosmetic_surgeon = df['speciality'].str.contains('|'.join(cosmetic_surgeon_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(cosmetic_surgeon_exact_matches)
mask_cosmetic_surgeon_exclusions = df['speciality'].isin(cosmetic_surgeon_exclusions)

# Final mask: Select cosmetic_surgeon
mask_cosmetic_surgeon_final = mask_cosmetic_surgeon & ~mask_cosmetic_surgeon_exclusions

# Store the original values that will be replaced
original_cosmetic_surgeon_values = df.loc[mask_cosmetic_surgeon_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_cosmetic_surgeon_final, 'speciality'] = 'Cosmetic Surgeon'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Cosmetic Surgeon", 'green'))
print(df.loc[mask_cosmetic_surgeon_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_cosmetic_surgeon_values = df.loc[mask_cosmetic_surgeon_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Cosmetic Surgeon", "cyan"))
for original_cosmetic_surgeon_value in original_cosmetic_surgeon_values:
    print(f"✅ {original_cosmetic_surgeon_value} → Cosmetic Surgeon")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Cosmetic Surgeon:", 'red'))
print(grouped_cosmetic_surgeon_values)

# Print summary
matched_count_cosmetic_surgeon = mask_cosmetic_surgeon_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Cosmetic Surgeon: "
        f"{matched_count_cosmetic_surgeon}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Pediatrician related titles

pediatrician_variants = [
    # Standard Titles & Variants
    r"(?i)\bPediatrician\b",
    r"(?i)\bPediatrist\b",
    r"(?i)\bPediatrics Specialist\b",
    r"(?i)\bPediatric Doctor\b",
    r"(?i)\bChild Specialist\b",
    r"(?i)\bPediatrics Physician\b",
    r"(?i)\bPediatric Consultant\b",
    r"(?i)\bPediatric Specialist\b",
    r"(?i)\bPediatric Practitioner\b",
    r"(?i)\bPediatric Surgeon\b",
    r"(?i)\bGeneral Pediatrician\b",
    r"(?i)\bPediatrician Consultant\b",
    r"(?i)\bConsultant Pediatrician\b",
    r"(?i)Paediatrics",
    r"(?i)\bPaediatric Surgery\b",
    r"(?i)\bPaediatrician\b",

    # Misspellings & Typographical Errors
    r"(?i)\bPediatrition\b",
    r"(?i)\bPediatrition\b",
    r"(?i)\bPedriatrician\b",
    r"(?i)\bPeditrician\b",
    r"(?i)\bPedatrician\b",
    r"(?i)\bPediatrion\b",
    r"(?i)\bPediatrian\b",

    # Case Variations
    r"(?i)\bpediatrician\b",
    r"(?i)\bPEDIATRICIAN\b",
    r"(?i)\bPeDiAtRiCiAn\b",
    r"(?i)\bPediatriCian\b",

    # Spanish Variants
    r"(?i)\bPediatra\b",
    r"(?i)\bPediatra\s?General\b",
    r"(?i)\bPediatra\s?Especialista\b",
    r"(?i)\bMédico\s?Pediatra\b",
    r"(?i)\bPediatra\s?Consultor\b",
    r"(?i)\bPediatra\s?Infantil\b",
    r"(?i)\bPediatra\s?Pediátrico\b",
    r"(?i)\bPediatra\s?Cirujano\b",
    r"(?i)\bPediatra\s?Generalista\b",

    # Other Possible Variations
    r"(?i)\bPediatric Physician\b",
    r"(?i)\bPediatric Consultant\b",
    r"(?i)\bPediatric Specialist\b",
    r"(?i)\bPediatric Doctor\b",
    r"(?i)\bChild Healthcare Professional\b",
    r"(?i)\bPediatric Surgeon\b",
]

# Exact matches that should be updated
pediatrician_exact_matches = {
    "Pediatrician",
    "Pediatrist",
    "Pediatrics Specialist",
    "Pediatric Doctor",
    "Child Specialist",
    "Pediatrics Physician",
    "Pediatric Consultant",
    "Pediatric Specialist",
    "Pediatric Practitioner",
    "Pediatric Surgeon",
    "General Pediatrician",
    "Pediatrician Consultant",
    "Consultant Pediatrician",
    "Pediatrtion",
    "Pediatrition",
    "Pedriatrician",
    "Peditrician",
    "Pedatrician",
    "Pediatrion",
    "Pediatrian",
    "Pediatra",
    "Pediatra General",
    "Pediatra Especialista",
    "Médico Pediatra",
    "Pediatra Consultor",
    "Pediatra Infantil",
    "Pediatra Pediátrico",
    "Pediatra Cirujano",
    "Pediatra Generalista",
    "Pediatric Physician",
    "Child Healthcare Professional",
}

# # Define patterns (these should NOT be changed)
# pediatrician_exclusions = r'\b(?:Plastic)|(?:Physician)\b'

pediatrician_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Pediatric Physician Recruiter',
}

# Create a mask for Pediatrician
mask_pediatrician = df['speciality'].str.contains('|'.join(pediatrician_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(pediatrician_exact_matches)

# mask_pediatrician_exclusions = df['speciality'].str.contains(pediatrician_exclusions, case=False, na=False, regex=True)
mask_pediatrician_exclusions = df['speciality'].isin(pediatrician_exclusions)

# Final mask: Select Pediatrician
mask_pediatrician_final = mask_pediatrician & ~mask_pediatrician_exclusions

# Store the original values that will be replaced
original_pediatrician_values = df.loc[mask_pediatrician_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_pediatrician_final, 'speciality'] = 'Pediatrician'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Pediatrician", 'green'))
print(df.loc[mask_pediatrician_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_pediatrician_values = df.loc[mask_pediatrician_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Pediatrician", "cyan"))
for original_pediatrician_value in original_pediatrician_values:
    print(f"✅ {original_pediatrician_value} → Pediatrician")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Pediatrician:", 'red'))
print(grouped_pediatrician_values)

# Print summary
matched_count_pediatrician = mask_pediatrician_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Pediatrician: "
        f"{matched_count_pediatrician}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Company Manager related titles

company_manager_variants = [
    r"(?i)\bcompany\s?manager\b",
    r"(?i)\bmanager\s?of\s?company\b",
    r"(?i)\bcompany\s?director\b",
    r"(?i)\bcorporate\s?manager\b",
    r"(?i)\bbusiness\s?manager\b",
    r"(?i)\bexecutive\s?manager\b",
    r"(?i)\bchief\s?manager\b",
    r"(?i)\bcompany\s?maneger\b",
    r"(?i)\bcompnay\s?manager\b",
    r"(?i)\bcompanny\s?manager\b",
    r"(?i)\bcomapny\s?manager\b",
    r"(?i)\bcomany\s?manager\b",
    r"(?i)\bcompany\s?managr\b",
    r"(?i)\bcompany\s?manegar\b",
    r"(?i)\bgerente\s?de\s?empresa\b",
    r"(?i)\bgestor\s?de\s?empresa\b",
    r"(?i)\bdirector\s?de\s?empresa\b",
    r"(?i)\badministrador\s?de\s?empresa\b",
    r"(?i)\bcompany\s?lead\b",
    r"(?i)\bcompany\s?head\b",
    r"(?i)\bmanaging\s?director\b",
    r"(?i)\bexecutive\s?director\b",
    r"(?i)\bsenior\s?manager\s?of\s?company\b",
    r"(?i)\bAssistant Manager\b",
    r"(?i)\bRegional Manager\b",
    r"(?i)\bGerente\b",
    r"(?i)\bTerritory Manager\b",
    r"(?i)\bArea Manager\b",
    r"(?i)\bAssociate Director\b",
    r"(?i)\bSenior Director\b",
    r"(?i)\bExhibitor\b",
    r"(?i)\bDirector Of Operations\b",
    r"(?i)\bOperations Director\b",
    r"(?i)\bGerant\b",
    r"(?i)\bCountry\s?Manager\b",
    r"(?i)\bOperations\b",
    r"(?i)\bSenior Manager\b",
    r"(?i)\bMedtech Engineering\b",
    r"(?i)\bImported Machines\b",
    r"(?i)\bExportmanager\b",
    r"(?i)\bBusiness Unit Manager\b",
    r"(?i)\bExec Assistant\b",
    r"(?i)\bCosmetic Manufacturer\b",
    r"(?i)\bExport Assistant\b",
    r"(?i)Export",
    r"(?i)Import",
    r"(?i)Manufacturer",
    r"(?i)\bHead Of Supplies\b",
    r"(?i)\bDistribution Engineer\b",
    r"(?i)\bMedical Industry\b",
    r"(?i)\bRegenova\b",
    r"(?i)\bSenior Business Services Manager\b",
    r"(?i)\bBusiness & Management\b",
    r"(?i)\bSenior Executive\b",
    r"(?i)\bQuality Manager\b",
    r"(?i)\bHead Of Asia Pacific\b",
    r"(?i)\bHead Manager\b",
    r"(?i)\bManpower Supply\b",
    r"(?i)Purchasing",
    r"(?i)\bDoo\b",
    r"(?i)\bMedical Manager\b",
    r"(?i)\bDeputy Head Of Ir\b",
    r"(?i)\bMba\b",
    r"(?i)\bOperation Manager\b",
    r"(?i)\bAccount Supervisor\b",
    r"(?i)\bPurchasing Officer\b",
    r"(?i)\bDeputyoperationmanager\b",
    r"(?i)\bBba In Finance\b",
    r"(?i)\bCompliance Manager\b",
    r"(?i)\bSpecial Envoy On Trade & Investment\b",
    r"(?i)\bInformation Systems Manager\b",
    r"(?i)\bFinance Manager\b",
    r"(?i)\bInternational Trade\b",
    r"(?i)\bInteriors Motifs\b",
    r"(?i)\bPurchaser\b",
    r"(?i)\bSupply Chain Manager\b",
    r"(?i)\bOperational Manager\b",
    r"(?i)\bHead Of Global Technical Support Gxr Solutions\b",
    r"(?i)\bCapital Markets\b",
    r"(?i)\bOlson Ltd\b",
    r"(?i)\bAmbassador Of Bioskin Dermolife\b",
    r"(?i)\bPharmaceuticals\b",
    r"(?i)\bHr Admin Or SalaryExpenses\b",
    r"(?i)\bLecturerTechnologist\b",
    r"(?i)\bUs Western Manager\b",
    r"(?i)\bHr\b",
    r"(?i)\bWholesale Buyer\b",
    r"(?i)\bManager Officer\b",
    r"(?i)\bPharmaceutical\b",
    r"(?i)\bMgr Exec Strategic Account - Skm\b",
    r"(?i)\bMember Manager\b",
    r"(?i)\bLogistic Agency For Relife\b",
    r"(?i)\bCustomer Relations Manager\b",
    r"(?i)\bBuyer\b",
    r"(?i)\bSenior Buyer\b",
    r"(?i)\bAi Application In Aesthetic Medicine\b",
    r"(?i)\bRelife France\b",
    r"(?i)\bGlobal Head Of Beauty\b",
    r"(?i)\bBanking\b",
    r"(?i)\bMeg Ambassador\b",
    r"(?i)\bAudit Manager\b",
    r"(?i)\bHead Of Corporate Finance & Development\b",
    r"(?i)\bBusiness Relations Manager\b",
    r"(?i)\bBuilder Company\b",
    r"(?i)\bEmerging Tech\b",
    r"(?i)\bAssociate Manager\b",
    r"(?i)\bAssociate Media Connections PlannerBuyer\b",
    r"(?i)\bInvestment Associate\b",
    r"(?i)\bBuyers\b",
    r"(?i)\bShipping-&-Receiving Supervisor\b",
    r"(?i)\bWarehouse Supervisor\b",
    r"(?i)\bSoftware\b",
    r"(?i)\bSkin Care Brand\b",
    r"(?i)\bSurgeon Recruiter\b",
    r"(?i)\bRecruiting Coordinator\b",
]

# Exact matches that should be updated
company_manager_exact_matches = {
    'Company Manager',
    'Company Director',
    'Corporate Manager',
    'General Manager of Company',
    'Business Manager',
    'Executive Manager',
    'Chief Manager',
    'Manager of Company',
    'Manager of the Company',
    'Manager',
    'Pharma Industry',
    'Retail Chains',
    'Specialized Store/Concept Store',
    'Industry',
    'Business',
    'Manger',
    'Executive',
    'Director Of Operations',
    'Operations Director',
    'Gerant',
    'Country Manager',
    'Operations',
    'Company',
    'Directeur',
    'Directeur',
    'Director',
    'director',
    'Senior Associate',
    'Sr  Associate',
    'Director Of Customer Successtset',
    'Girector General',
    'Medical Equipment',
    'Purchase Specialist',
    'Travel Coordinator',
    'Travel Management',
    'Travel Agent',
    'Director Finance Treasury',
    'Director Of Strategic Partnerships',
    'Director Client Partnerships',
    'Director Of Information Technology',
    'Middle East Director',
    'Us Director',
    'Director Of Corp Development',
    'Country Director',
    'Director Of Professional Relations',
    'Medical Contracts Director',
    'Ict Director',
    'Board Director',
    'Manager Director',
    'Sr  Director',
    'Managing Directo',
    'Business Team Director',
    'Director Of Corporate Relations',
    'Overseas Business Department Manager',
    'International Director',
    'Medytox',
    'Technology Transfer Director',
    'Manager Of New Business Segments',
    'Direcctor',
    'Medical Company',
    'Managers',
    'Director Finance',
    'Ict Manager',
    'Sr  Manager Partner Relations',
    'Opperations',
    'Enhibitor',
    'Managing Dir Shield Network',
    'District Manager',
    'Managing Dir Shield Network',
    'Director Of International Cooperation',
    'International Relations',
    'Fillmed Greece',
    'Group Business Director',
    'Sustainability Executive',
    'Biotechnology Hydrogel',
    'Information Security Officer',
    'Corporate Management',
    'Customer Solution Officer',
    'Global  Latin America',
    'Managing DirectorFounder',
    'Regional Head Of Medical Equipment',
    'Head Of Information Technology',
    'Information Technology Director',
    'Co-Director & Manager',
    'DIRECTOR',
    'People & Culture Manager',
    'Specialized Store Concept Store',
    'Board Member',
    'Board of Director',
    'CPA - Treasurer',
    'Director HCP Advocacy',
    'Director Industry Outreach',
    'Director Latin America & Caribbean Islands',
    'Director of Customer Engagement',
    'Director of LATAM Relations',
    'Director of Partnerships',
    'Diretora',
    'District Manager SkinMedica',
    'District Manager FA',
    'Head of Finance',
    'MAC Director',
    'Management Staff',
    'Manager MAX Platform',
    'Manegment Staff',
    'Mgr',
    'Mgr Sr Dev - BOTOX Cosmetic',
    'Premier',
    'Recens USA',
    'Senior Acct Exec',
    'Treasurer & Director',
    'Director Mergers & Acquisitions',
    'Sr Director - Medical Aesthetics Gbu',
    'Fractional Executive',
    'Talent Acquisition- Dentist& Field Staff',
    'Director Dermatology Business',
    'Director Dermatology Business Texas & Ok North',
    'Director Of Physician Compensation',
    'District Manager- Dermatology',
    'Manager Information Technology At Md Alliance Solutions Llc Advanced Dermatology Pc',
    'Physician Finance Officer',
    'Site Manager - General Surgery',
    'Financial Assistant',
    'ASM North Texas',
    'Area Director',
    'Asset Strategy Leader Aesthetics',
    'Assoc Director Search & Evaluation Aesthetics',
    'Asst Mgr',
    'B S',
    'BA',
    'BBA',
    'BENEV',
    'BS',
    'Bachelor',
    'Bachelor of Science in Business',
    'Big D',
    'CPO',
    'Chief Accounts Officer',
    'Chief Culture Coordinator',
    'Chief Evangelist',
    'Chief Growth Officer',
    'Chief Revenue Off',
    'Chief Strategy Officer',
    'Chief Revenue Officer Of Buildmyhealth',
    'Chief Technology officer',
    'Client Support Manager',
    'Corporate Account Director',
    'Corporate Accounts Manager',
    'Cosmetic Peel Company',
    'Customer Success Manager',
    'Customer Success Specialist',
    'Customer Success Team',
    'CRO',
    'Dir Strat Accts',
    'Dir Strategic Accounts',
    'Director Ax Customer Engagement -East',
    'Director Biostimulators',
    'Director Clinical Development Allergan Aesthetic Medicine',
    'Director Corporate Accounts',
    'Director Customer Engagement',
    'Director Customer Engagement -West',
    'Director Customer Engagement East',
    'Director Customer Loyalty',
    'Director Development & Acquisitions',
    'Director External Innovation',
    'Director Field Service Engineer',
    'Director Finance Accounting',
    'Director National Accounts',
    'Director Of Strategic Accounts',
    'Director Portfolio & Strategic Implementation',
    'Director Portfolio Programs',
    'Director Portfolio Programs & National Conventions',
    'Director Strategic Accounts & Physicians',
    'Director Strategic Accounts - Medic',
    'Director Strategic Accounts - Medica',
    'Director Technical Services',
    'Director of Customer Affairs',
    'Director of Customer Experience',
    'Director of National Accounts',
    'Director of Recruitment',
    'District Manager FA- Austin TX',
    'District Manager- National Accounts',
    'Emerald Specialist',
    'Exec Dir Professional Relations',
    'Exetcutive',
    'Experience Manager',
    'Financial Director',
}

# Define patterns (these should NOT be changed)
company_manager_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Marketing Director',
    'Operations Project Manager',
    'Registered Nurse Program Manager-Clinical Operations',
    'Director Of Clinical Operations',
    'Rn Bsn Phn Mba',
    'Chief Of Operations & Practice Manager',
    'RN BSN MBA'
 }

# Create a mask for Company Manager
mask_company_manager = df['speciality'].str.contains('|'.join(company_manager_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(company_manager_exact_matches)
mask_company_manager_exclusions = df['speciality'].isin(company_manager_exclusions)

# Final mask: Select Company Manager
mask_company_manager_final = mask_company_manager & ~mask_company_manager_exclusions

# Store the original values that will be replaced
original_company_manager_values = df.loc[mask_company_manager_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_company_manager_final, 'speciality'] = 'Company Manager'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Company Manager", 'green'))
print(df.loc[mask_company_manager_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_company_manager_values = df.loc[mask_company_manager_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Company Manager", "cyan"))
for original_company_manager_value in original_company_manager_values:
    print(f"✅ {original_company_manager_value} → Company Manager")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Company Manager:", 'red'))
print(grouped_company_manager_values)

# Print summary
matched_count_company_manager = mask_company_manager_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Company Manager: "
        f"{matched_count_company_manager}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for biologist_biochemist related titles

biologist_biochemist_variants = [
    r"(?i)\bBiologist\b",
    r"(?i)\bBiochemist\b",
    r"(?i)\bBiologist\s?Doctor\b",
    r"(?i)\bBiochemist\s?Doctor\b",
    r"(?i)\bBiologist\s?Specialist\b",
    r"(?i)\bBiochemist\s?Specialist\b",
    r"(?i)\bBiologist\s?Expert\b",
    r"(?i)\bBiochemist\s?Expert\b",
    r"(?i)\bBiology\s?Doctor\b",
    r"(?i)\bBiochemistry\s?Doctor\b",
    r"(?i)\bBiology\s?Specialist\b",
    r"(?i)\bBiochemistry\s?Specialist\b",
    r"(?i)\bFormulator\b",
    r"(?i)\bBiologist/Biochemist\b",
    r"(?i)\bBiologistBiochemist\b",
    r"(?i)\bBiologia Molecular E Estetica\b",
    r"(?i)\bBiotechnology Skincare\b",
    r"(?i)\bMicrobiome\b",
    r"(?i)\bBioanalytiker\b",
    r"(?i)\bBioquimica\b",
    r"(?i)\bBitoulogue\b",
    r"(?i)\bChemistry\b",
]

# Exact matches that should be updated
biologist_biochemist_exact_matches = {
    'Biologist',
    'Biochemist',
    'Biologist Doctor',
    'Biochemist Doctor',
    'Biologist/Biochemist',
    'BiologistBiochemist',
    'Formulator',
    'Biologist Specialist',
    'Biochemist Specialist',
    'Biologist Expert',
    'BiologistBiochemist',
    'Biochemist Expert',
    'Biology Doctor',
    'Biochemistry Doctor',
    'Biology Specialist',
    'Biochemistry Specialist',
    'Biology Physician',
    'Biochemistry Physician',
    'Biochemist MD',
    'Biologist MD',
    'PhD Biologist',
    'PhD Biochemist',

    # Case-related errors
    'biologist',
    'biochemist',
    'BIOLOGIST',
    'BIOCHEMIST',
    'BiOlOgIsT',
    'BiOcHeMiSt',
    'biology doctor',
    'biochemistry doctor',
    'biology specialist',
    'biochemistry specialist',

    # Common misspellings
    'Bioligist',
    'Bilogist',
    'Biochemest',
    'Biochemis',
    'Biologyist',
    'Bilogist',
    'Biologist Doctorate',
    'Biochemist Docter',
    'Bilogist Doctor',
    'Bioochemist',
    'Biyochemist',

    # Spanish-related exclusions
    'Biólogo',
    'Bióloga',
    'Bioquímico',
    'Bioquímica',
    'Doctor en Biología',
    'Doctor en Bioquímica',
    'Especialista en Biología',
    'Especialista en Bioquímica',
    'Médico Biólogo',
    'Médico Bioquímico',
    'Biólogo especialista',
    'Bioquímico especialista',
    'G MBiologist',
    'Medecin Biologiste',
    'Medical Biotechnologist',
    'Biolog',
}

# Define patterns for  & Resident & Professor (these should NOT be changed)
biologist_biochemist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Biologist Biochemist
mask_biologist_biochemist = df['speciality'].str.contains('|'.join(biologist_biochemist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(biologist_biochemist_exact_matches)
mask_biologist_biochemist_exclusions = df['speciality'].isin(biologist_biochemist_exclusions)

# Final mask: Select Biologist Biochemist
mask_biologist_biochemist_final = mask_biologist_biochemist & ~mask_biologist_biochemist_exclusions

# Store the original values that will be replaced
original_biologist_biochemist_values = df.loc[mask_biologist_biochemist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_biologist_biochemist_final, 'speciality'] = 'Biologist Biochemist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Biologist Biochemist", 'green'))
print(df.loc[mask_biologist_biochemist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_biologist_biochemist_values = df.loc[mask_biologist_biochemist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Biologist Biochemist", "cyan"))
for original_biologist_biochemist_value in original_biologist_biochemist_values:
    print(f"✅ {original_biologist_biochemist_value} → Biologist Biochemist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Biologist Biochemist:", 'red'))
print(grouped_biologist_biochemist_values)

# Print summary
matched_count_biologist_biochemist = mask_biologist_biochemist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Biologist Biochemist: "
        f"{matched_count_biologist_biochemist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for aesthetics_antiaging_physician related titles
aesthetic_plastic_surgeon_variants = [
    r"(?i)\bAesthetic & Plastic Surgeon\b",
    r"(?i)\bChirurgien Esthetique\b",
    r"(?i)\bEsthetic Surgeon\b",
    r"(?i)\bAesthetic Surgery\b",
    r"(?i)\bDermatosurgery\b",
    r"(?i)\bPlastic Surgery & Aesthetic Medicine\b",
    r"(?i)\bCraniofacial Aesthetic & Plastic Surgeon\b",
    r"(?i)\bAesthetic Surgeon\b",
    r"(?i)\bPlastic & Aesthetic Surgeon\b",
    r"(?i)\bMedico EsteticistaCirurgia Plastica\b",
    r"(?i)\bPlastic Arsthetics\b",
    r"(?i)\bDermatologist Cosmetologist Plastic Surgery\b",
    r"(?i)\bConsultant Plastic & Aesthetic Surgeon\b",
    r"(?i)\bPlastic Surgery Aesthetic Surgery\b",
    r"(?i)\bAesthetic Plastic Surgeon\b",
    r"(?i)\bCosmetology & Plastic Surgery\b",
    r"(?i)\bPlastic Operations\b",
    r"(?i)\bChirurgie Esthetique\b",
    r"(?i)\bCirujano Estetico\b",
    r"(?i)\bMedico CirujanoMedico Esteticista\b",
    r"(?i)\bOtorrinolaringologo Y Cirujano Estetico\b",
    r"(?i)\bMedica De Cirurgia Estetica\b",
    r"(?i)\bMedico Cirujano Estetico Medico Estetico Antienvegesimiento\b",
    r"(?i)\bFacharzt Chirurgie Und Asthetische Medizin\b",
]

# Exact matches that should be updated

aesthetic_plastic_surgeon_exact_matches = {
    # Existing exact matches for exclusions
    'Aesthetic & Plastic Surgeon',
    'Aesthetic Surgery',
    'Aesthetic Chirurgien',
    'Chirurgien Esthetique',
    'Esthetic Chirurgeon',
    'Aesthetic Surgeon',
    'Aesthetic Surgery Injector',
    'Aesthetic Plastic Surgery Injector',
    'Plastic Arsthetics',
    'Dermatosurgery',
    'Surgeons of Aesthetic Plastic Surgery',
    'Restorative Aesthetic Surgery',
    'Chirurgienne Esthétique',
    'Aesthetic & Plastic Surgeon',

    # Variations for misspellings and case mistakes for Aesthetic Plastic Surgeon and related titles
    'Aesthetic & Platic Surgeon',
    'Aesthetic Surgeons',
    'Aesthetic Plastic Surgeons',
    'Plastic Surgeon Aesthetic',
    'Aesthetic Surgery Specialist',
    'Aesthetic Surgical Specialist',
    'Plastic Surgery Aesthetic',
    'Plastic Aesthetic Surgeon',
    'Aesthetic Plastic Surgeon Specialist',
    'Aesthetic Surgeon Specialist',
    'Plastic Surgeons Aesthetic',
    'Plastic Surgeons of Aesthetic Surgery',
    'Aesthetician Plastic Surgeon',
    'Aesthetic Surgery Specialist',
    'Surgical Aesthetic Surgeon',
    'Aesthetic Plastic Surgery Specialist',
    'Aesthetic Surgery Surgeons',
    'Aesthetic Surgeon Doctor',
    'Plastic Surgeon Aesthetic Specialist',
    'Aesthetic Surgical Doctor',
    'Aesthetic Surgery Doctor',
    'Plastic Surgery Doctor Aesthetic',

    # Common misspellings for Aesthetic Plastic Surgery titles
    'Aesthetic Pplastic Surgeon',
    'Aestheti Surgery',
    'Aesthetic Pastic Surgeon',
    'Aestehtic Surgeon',
    'Aesthetic Surgen',
    'Aesthetic Plastc Surgeon',
    'Aesthetic Surgon',
    'Aesthetic Plastic Surgon',
    'Aesthtic Surgeon',
    'Plastic Arsthetics Specialist',
    'Aesthetic Surgen',
    'Aesthetic & Platic Surgeon',

    # Case-related errors for Aesthetic Plastic Surgeon
    'aesthetic & Plastic surgeon',
    'Aesthetic Plastic surgeon',
    'Aesthetic surgeron',
    'aesthetic surgery injector',
    'plastic Aesthetic surgeon',
    'aesthetic and plastic Surgeon',
    'plastic Surgeon aesthetic',
    'aesthetic surgery injector',
    'plastic & Aesthetic surgeon',
    'Plastic Surgeon aesthetic',

    # Variants of Aesthetic Surgeon for different types of surgeries
     'Aesthetic Surgery Body Surgeon',
    'Aesthetic Body Plastic Surgeon',

    # Other possible variations (Including Doctor forms, Specialist forms)
    'Aesthetic Plastic Surgery Doctor',
    'Doctor Aesthetic Plastic Surgeon',
    'Aesthetic & Plastic Surgeon Doctor',
    'Doctor of Aesthetic Surgery',
    'Aesthetic Surgical Doctor',
    'Doctor of Aesthetic Plastic Surgery',

    # Excluding Aesthetic Plastic Surgery titles with variations of surgeons
    'Aesthetic Plastic Surgeon Specialist',
    'Aesthetic Plastic Surgery Consultant',
    'Plastic Surgery Cosmetic Specialist',
    'Aesthetic Surgery Specialist',
    'Aesthetic Surgeon & Specialist',
    'International Exchange & Cooperation Department Of China Plastic & Aesthetic Association',
    'Aestethic surgeon',
}

# Define patterns for Maxillo & Facial Plastic & Resident (these should NOT be changed)
aesthetic_plastic_surgeon_exclusions = {
    # Existing exact matches for exclusions
    'Maxillo Facial Plastic Surgery',
    'Resident Plastic Surgery',
    'Facial Plastic Surgery',
    'Head Of The Department Of Facial & Plastic Surgery',
    'Head Of Department Of Plastic Surgery',
    'Cosmetic Doctor Plastic Surgery Resident',
    'Facial Aesthetics Plastic Surgery',
    'Resident In Plastic Surgery',
    'Assistant Professor In Plastic Surgery/Hawler Me',
    'Professor Of Plastic Surgery',
    'Associate Professor Of Plastic Surgery',
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Plastic & Reconstructive Surgeon',
    'Plastic Surgery Medical Assistant',
    'Rnbn Clinic Director Plastic Surgery Centre',
    'Coo Of A Clinic For Plastic Surgery',
    'Rep Sr Plastic Surgery Us Plastice Surgery',

    # Variations for misspellings and case mistakes for Maxillo/Facial/Plastic Surgery
    'Maxillofacial Plastic Surgery',
    'Maxillo-Facial Plastic Surgery',
    'Facial Aesthetic Plastic Surgery',
    'Maxillofacial Surgery Resident',
    'Resident Facial Surgery',
    'Plastic Surgeon Resident',
    'Facial Plastic Surgeon Resident',
    'Plastic Surgery Resident',
    'Maxillofacial Surgeon Resident',
    'Maxillo Facial Surgery Resident',
    'Maxillofacial Plastic Surgery Fellow',
    'Resident Maxillofacial Surgeon',
    'Cosmetic Surgeon Resident',
    'Plastic Surgery Intern',
    'Resident Surgeon (Plastic Surgery)',
    'Assistant Professor Maxillofacial Surgery',
    'Assistant Professor Plastic Surgery',
    'Facial Plastic Surgery Fellow',
    'Fellow in Maxillofacial Surgery',
    'Junior Maxillofacial Surgeon',
    'Plastic Surgery Fellow',
    'Professor Plastic Surgery',
    'Plastic Surgery Trauma Co-Ordinator',

    # Variants of student/professor related titles:
    'Plastic Surgery Student',
    'Student in Plastic Surgery',
    'Clinical Fellow Maxillofacial Surgery',
    'Junior Plastic Surgeon',
    'Fellow in Facial Plastic Surgery',
    'Facial Surgeon Fellow',
    'Trainee Plastic Surgeon',
    'Resident Surgeon Maxillofacial',
    'Intern Plastic Surgery',
    'Plastic Surgery Postgraduate',
    'Trainee Aesthetic Plastic Surgeon',
    'Postgraduate Fellow Plastic Surgery',
    'Fellow Aesthetic Surgery',
    'Plastic Surgery Resident Trainee',
    'Clinical Fellow Plastic Surgery',
    'Maxillofacial Surgery Resident',
    'Facial Surgery Intern',
    'Facial Surgeon Resident',
    'Junior Facial Surgeon',
    'Professor Maxillofacial Surgery',
    'Assistant Plastic Surgery Professor',
    'Junior Facial Plastic Surgeon',
    'Facial Surgery Postgraduate Trainee',

    # Common misspellings and alternative phrasings
    'Maxillo-Facial Surgeon',
    'Maxillofacial Surgeion',
    'Facial Surgeon',
    'Plastic Surgeion',
    'Facial Surgeron',
    'Maxillofaciel Surgery',
    'Maxillofacial Surgery',
    'Facial Plastc Surgery',
    'Facial Plastc Surgeon',
    'Maxilofacial Surgery',
    'Maxillofacial Surrgery',
    'Facial Surgery Specalist',
    'Facial Surgery Practitionar',
    'Facial Plastic Surgeon',
    'Facial Surgery Doctor',
    'Maxillofacial Plastician',
    'Maxillofacial Aesthetician',
    'Facial Surgery Resident',
    'Facial Plastic Surgeon Resident',
    'Maxillofacial Practitioner',
    'Aesthetic Maxillofacial Surgeon',
    'Junior Facial Plastic Surgery',

    # Case-related errors
    'Maxillo Facial Plastic Surgeonn',
    'Facial Platic Surgery',
    'Maxillo Face Surgery',
    'Plastic Surgon Resident',
    'Maxillo-Facial Plastic Surgon',
    'Facial Plastics Surgeion',
    'Fellow in Maxillo Facial Surgery',
    'Resident Facial Surgeon',
    'Maxillofacial Aesthetic Surgeon',
    'Resident Facail Surgeon',
    'Maxillofacial Plastic Surgery Fellow',
    'Maxillofacial Surgery Fellow',
    'Aesthetic Facail Surgeon',
    'Maxillofacial Plastician Resident',
    'Plastic Surgery Pa',

    # Additional Facial Plastic Surgeon-related titles:
    'Facial Plastic Surgeon',
    'Facial Reconstructive Surgeon',
    'Facial Plastic Surgeon Specialist',
    'Plastic & Reconstructive Surgeon',
    'Plastic Reconstructive & Aesthetic Surgeon',
    'Reconstructive Facial Plastic Surgeon',
    'Reconstructive Surgeon',
    'Cosmetic Facial Plastic Surgeon',
    'Cosmetic & Reconstructive Facial Surgeon',
    'Plastic & Reconstructive Surgeon',
    'Plastic & Reconstructive Surgery Fellow',
    'Facial Aesthetic Plastic Surgeon',
    'Facial Aesthetic Surgeon',
    'Facial Plastic Surgery Specialist',
    'Facial Cosmetic Surgeon',
    'Plastic Surgeon (Facial Surgery)',
    'Aesthetic Plastic & Reconstructive Surgeon',
    'Aesthetic Facial Plastic Surgeon',
    'Cosmetic Facial Surgeon',
    'Reconstructive Plastic Surgeon',
    'Facial Plastic Surgeon (Cosmetic)',
    'Facial Plastic Surgery Doctor',

    # Misspellings and case-related errors for Facial Plastic Surgeons:
    'Facial Plastc Surgeon',
    'Facial Surgeoon',
    'Plastic Surgion (Facial)',
    'Facial Surrgeon',
    'Facial Plastic Surgery Doctor',
    'Facial Plastic Surgon',
    'Cosmetic Facial Surgeon',
    'Plastic & Reconstructive Surgeion',
    'Reconstructive Facial Surgeion',
    'Plastic Reconstructive Surgeion',
    'Facial Plastic Surgeon (Reconstructive)',
    'Facial Cosmetic Surgeron',
    'Facial Plastic Surgery Spcialist',
    'Senior Resident Department Of Plastic Surgery',
    'Rep Plastic Surgery Us Plastic Surgery',
    'Plastic Rekonstructive & Aesthetic Surgery',
    'Plastic Reconstructive & Aesthetic Surgery',
    'Consultant Facial Plastic Surgeon',
    'Plastic Reconstructive Aesthetic Surgery',
}

# Create a mask for aesthetic_plastic_surgeon
mask_aesthetic_plastic_surgeon = df['speciality'].str.contains('|'.join(aesthetic_plastic_surgeon_variants), case=False, na=False, regex=True) | \
            df['speciality'].isin(aesthetic_plastic_surgeon_exact_matches)
mask_aesthetic_plastic_surgeon_exclusions = df['speciality'].isin(aesthetic_plastic_surgeon_exclusions)

# Final mask: Select Aesthetic & Plastic Surgeon values
mask_aesthetic_plastic_surgeon_final = mask_aesthetic_plastic_surgeon & ~mask_aesthetic_plastic_surgeon_exclusions

# Store the original values that will be replaced
original_aesthetic_plastic_surgeon_values = df.loc[mask_aesthetic_plastic_surgeon_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_aesthetic_plastic_surgeon_final, 'speciality'] = 'Aesthetic & Plastic Surgeon'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Aesthetic & Plastic Surgeon", 'green'))
print(df.loc[mask_aesthetic_plastic_surgeon_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_aesthetic_plastic_surgeon_values = df.loc[mask_aesthetic_plastic_surgeon_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Aesthetic & Plastic Surgeon", "cyan"))
for original_aesthetic_plastic_surgeon_value in original_aesthetic_plastic_surgeon_values:
    print(f"✅ {original_aesthetic_plastic_surgeon_value} → Aesthetic & Plastic Surgeon")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Aesthetic & Plastic Surgeon:", 'red'))
print(grouped_aesthetic_plastic_surgeon_values)

# Print summary
matched_count_aesthetic_plastic_surgeon = mask_aesthetic_plastic_surgeon_final.sum()

# Print results
print(colored(f"\nTotal values matched and changed (Stage 1) to Aesthetic & Plastic Surgeon: {matched_count_aesthetic_plastic_surgeon}", 'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Orthopedist related titles

orthopedist_variants = [
    # Standard Titles & Abbreviations
    r"(?i)\bOrthopedist\b",
    r"(?i)\bOrthopedic\s?Doctor\b",
    r"(?i)\bOrthopedic\s?Surgeon\b",
    r"(?i)\bOrtho\s?Doctor\b",
    r"(?i)\bOrtho\s?Specialist\b",
    r"(?i)\bOrthopedics\s?Practitioner\b",
    r"(?i)\bOrthopedics\s?Consultant\b",
    r"(?i)\bMD\s?Orthopedics\b",
    r"(?i)\bDMD\s?Orthopedics\b",
    r"(?i)\bProsthetist & Orthotist\b",
    r"(?i)\bTraumatology\b",

    # Misspellings & Typographical Errors
    r"(?i)\bOrthopaedist\b",
    r"(?i)\bOrtopedist\b",
    r"(?i)\bOrthopediest\b",
    r"(?i)\bOrthepedist\b",
    r"(?i)\bOrthopeedist\b",
    r"(?i)\bOrthopedics\s?Doctor\b",

    # Case Variations
    r"(?i)\borthopedist\b",
    r"(?i)\bOrthopedIst\b",
    r"(?i)\bORTHOPEDIST\b",
    r"(?i)\bOrThOpEdIsT\b",

    # Spanish Variants
    r"(?i)\bOrtopedista\b",
    r"(?i)\bDoctor\s?en\s?Ortopedia\b",
    r"(?i)\bCirujano\s?Ortopédico\b",
    r"(?i)\bEspecialista\s?en\s?Ortopedia\b",

    # Other Possible Variations (Including Doctor forms, Specialist forms)
    r"(?i)\bOrthopedic\s?Expert\b",
    r"(?i)\bOrthopedic\s?Consultant\b",
    r"(?i)\bSpecialist\s?in\s?Orthopedics\b",
    r"(?i)\bOrthopedics\s?Surgeon\b",
]

# Exact matches that should be updated
orthopedist_exact_matches = {
    "Orthopedist",
    "Orthopedic Doctor",
    "Orthopedic Surgeon",
    "Ortho Doctor",
    "Ortho Specialist",
    "Orthopedics Practitioner",
    "Orthopedics Consultant",
    "MD Orthopedics",
    "DMD Orthopedics",
    "Orthopaedist",
    "Ortopedist",
    "Orthopediest",
    "Orthepedist",
    "Orthopeedist",
    "Orthopedics Doctor",
    "Ortopedista",
    "Doctor en Ortopedia",
    "Cirujano Ortopédico",
    "Especialista en Ortopedia",
    "Orthopedic Expert",
    "Orthopedic Consultant",
    "Specialist in Orthopedics",
    "Orthopedics Surgeon",
    'Orthopedic',
    'orthopedic',
    'Orthopedics',
    'orthopedics',
    'Consultant Orthopaedics',
    'Consultant Trauma & Orthopaedic Surgery',
    'KieferorthopadieAsthetik',
    'Orthopaedics',
}

# # Define patterns (these should NOT be changed)
# orthopedist_exclusions = r'\b(?:Plastic)|(?:Physician)\b'

orthopedist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Orthopedist
mask_orthopedist = df['speciality'].str.contains('|'.join(orthopedist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(orthopedist_exact_matches)

# mask_orthopedist_exclusions = df['speciality'].str.contains(orthopedist_exclusions, case=False, na=False, regex=True)
mask_orthopedist_exclusions = df['speciality'].isin(orthopedist_exclusions)

# Final mask: Select Orthopedist
mask_orthopedist_final = mask_orthopedist & ~mask_orthopedist_exclusions

# Store the original values that will be replaced
original_orthopedist_values = df.loc[mask_orthopedist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_orthopedist_final, 'speciality'] = 'Orthopedist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Orthopedist", 'green'))
print(df.loc[mask_orthopedist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_orthopedist_values = df.loc[mask_orthopedist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Orthopedist", "cyan"))
for original_orthopedist_value in original_orthopedist_values:
    print(f"✅ {original_orthopedist_value} → Orthopedist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Orthopedist:", 'red'))
print(grouped_orthopedist_values)

# Print summary
matched_count_orthopedist = mask_orthopedist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Orthopedist: "
        f"{matched_count_orthopedist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Plastic and Reconstructive Surgeon related titles

reconstructive_variants = [
    # Standard Titles & Variants
    r"(?i)\bPlastic and Reconstructive Surgeon\b",
    r"(?i)\bReconstructive Surgeon\b",
    r"(?i)\bAesthetic and Reconstructive Surgeon\b",
    r"(?i)\bCosmetic and Reconstructive Surgeon\b",
    r"(?i)\bReconstructive and Aesthetic Surgeon\b",
    r"(?i)Plastic & Reconstructive Surgery",
    r"(?i)\bOncoplastic Surgeon\b",
    r"(?i)\bPlastic Rekonstructive & Aesthetic Surgery\b",
    r"(?i)\bPlastic Reconstructive Aesthetic Surgery\b",
    r"(?i)\bChirurgie Plastique Et Reparatrice\b",

    # Misspellings & Typographical Errors
    r"(?i)\bPlasic and Reconstructive Surgeon\b",
    r"(?i)\bPlastic and Reconstuctive Surgeon\b",
    r"(?i)\bPlastic and Reconstructive Surgeoon\b",
    r"(?i)\bPlstic and Reconstructive Surgeon\b",
    r"(?i)\bPlstic and Reconstructive Surgon\b",
    r"(?i)\bPlastic and Reconstructuve Surgeon\b",

    # Case Variations
    r"(?i)\bplastic and reconstructive surgeon\b",
    r"(?i)\bPLASTIC AND RECONSTRUCTIVE SURGEON\b",
    r"(?i)\bPlAsTiC aND ReCoNsTrUcTiVe SuRgEoN\b",

    # Spanish Variants
    r"(?i)\bCirujano Plástico y Reconstructivo\b",
    r"(?i)\bCirujano Estético y Reconstructivo\b",
    r"(?i)\bCirujano Reconstructivo\b",

    # Other Possible Variations
    r"(?i)\bCosmetic Reconstructive Surgeon\b",
    r"(?i)\bReconstructive Surgery Specialist\b",
    r"(?i)\bReconstructive and Cosmetic Surgeon\b",
    r"(?i)\bPlastic & Reconstractive Surgeon\b",
    r"(?i)\bArmonizacion Orofacial\b",
    r"(?i)\bPlastic Aesthetic & Reconstructive Surgery\b",
    r"(?i)\bPlastic Reconstructive & Aesthetic Surgeon\b",
    r"(?i)\bPlastic Rekonstructive & Aesthetic Surgery\b",
    r"(?i)\bPlastic Reconstructive & Aesthetic Surgery\b",
    r"(?i)\bPlastic Reconstructive Aesthetic Surgery\b",
    r"(?i)\bSpecialist In Facial Harmonization\b",
]

# Exact matches that should be updated
reconstructive_exact_matches = {
    "Plastic and Reconstructive Surgeon",
    "Reconstructive Surgeon",
    "Aesthetic and Reconstructive Surgeon",
    "Cosmetic and Reconstructive Surgeon",
    "Reconstructive and Aesthetic Surgeon",
    "Plasic and Reconstructive Surgeon",
    "Plastic and Reconstuctive Surgeon",
    "Plastic and Reconstructive Surgeoon",
    "Plstic and Reconstructive Surgeon",
    "Plstic and Reconstructive Surgon",
    "Plastic and Reconstructuve Surgeon",
    "Cirujano Plástico y Reconstructivo",
    "Cirujano Estético y Reconstructivo",
    "Cirujano Reconstructivo",
    "Cosmetic Reconstructive Surgeon",
    "Reconstructive Surgery Specialist",
    "Reconstructive and Cosmetic Surgeon",
    'Plastic & Reconstrucitve Surgery',
    'Cirujana Plastica Y Reconstructiva'
}

# # Define patterns (these should NOT be changed)
# reconstructive_exclusions = r'\b(?:Plastic)|(?:Physician)\b'

reconstructive_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'A Professor Plastic & Reconstructive Surgery',
}

# Create a mask for Plastic and Reconstructive Surgeon
mask_reconstructive = df['speciality'].str.contains('|'.join(reconstructive_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(reconstructive_exact_matches)

# mask_reconstructive_exclusions = df['speciality'].str.contains(reconstructive_exclusions, case=False, na=False, regex=True)
mask_reconstructive_exclusions = df['speciality'].isin(reconstructive_exclusions)

# Final mask: Select Plastic and Reconstructive Surgeon
mask_reconstructive_final = mask_reconstructive & ~mask_reconstructive_exclusions

# Store the original values that will be replaced
original_reconstructive_values = df.loc[mask_reconstructive_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_reconstructive_final, 'speciality'] = 'Plastic and Reconstructive Surgeon'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Plastic and Reconstructive Surgeon", 'green'))
print(df.loc[mask_reconstructive_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_reconstructive_values = df.loc[mask_reconstructive_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Plastic and Reconstructive Surgeon", "cyan"))
for original_reconstructive_value in original_reconstructive_values:
    print(f"✅ {original_reconstructive_value} → Plastic and Reconstructive Surgeon")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Plastic and Reconstructive Surgeon:", 'red'))
print(grouped_reconstructive_values)

# Print summary
matched_count_reconstructive = mask_reconstructive_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Plastic and Reconstructive Surgeon: "
        f"{matched_count_reconstructive}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for General Surgeon related titles

general_surgeon_variants = [
    r"(?i)\bGeneral\s?Surgeon\b",
    r"(?i)\bGeneral\s?Surgeon\s?Specialist\b",
    r"(?i)\bBoard\s?Certified\s?General\s?Surgeon\b",
    r"(?i)\bCertified\s?General\s?Surgeon\b",
    r"(?i)\bGeneral Surgery\b",

    # Spanish variants
    r"(?i)\bCirujano\s?General\b",
    r"(?i)\bCirujano\s?Generalista\b",
    r"(?i)\bCirujano\s?de\s?Atención\s?Primaria\b",

    # Other possible variations
    r"(?i)\bDoctor\s?General\s?Surgeon\b",
    r"(?i)\bDoctor\s?en\s?Cirugía\s?General\b",
    r"(?i)\bGeneral\s?Surgical\s?Specialist\b",
    r"(?i)\bGeneral\s?Surgical\s?Doctor\b",
    r"(?i)\bGeneral\s?Surgical\s?Physician\b",
    r"(?i)\bLicensed\s?General\s?Surgeon\b",
    r"(?i)\bTrauma Physician\b",
    r"(?i)\bConsultant Surgeon\b",
    r"(?i)\bGeneral Suegery\b",
    r"(?i)\bConsultant General Sugeon\b",
    r"(?i)\bGeneral & Aesthetic Surged\b",
    r"(?i)\bMedico Cirugiao Geral\b",
    r"(?i)\bChief Surgeon\b",
    r"(?i)\bChief Of Surgery\b",
    r"(?i)\bSurgical & Diagnostics\b",
    r"(?i)\bConsultanttraumasurgeon\b",
    r"(?i)\bOhner The Surgery\b",
    r"(?i)\bChirurgie Generale Et Senologique\b",
    r"(?i)\bChirurgie Main\b",
    r"(?i)\bCirujano General\b",
    r"(?i)\bGeneral Surgeon\b",
    r"(?i)\bConsultant General Sugeon\b",
    r"(?i)\bConsultant Surgeon\b",
    r"(?i)\bChief Surgeon\b",
    r"(?i)\bGeneral Surgery\b",
    r"(?i)\bSurgical & Diagnostics\b",
    r"(?i)\bChief Of Surgery\b",
    r"(?i)\bGeneral Surgeon Facial Surgery Breast Surgery\b",
    r"(?i)\bConsultanttraumasurgeon\b",
    r"(?i)\bOhner The Surgery\b",
    r"(?i)\bChirurgie Generale Et Senologique\b",
    r"(?i)\bGeneral & Aesthetic Surged\b",
    r"(?i)\bChirurgie Main\b",
    r"(?i)\bGeneral Surgeon Specialized In Abdominal Wall Reconstruction\b",
    r"(?i)\bTrauma Physician\b",
    r"(?i)\bMedico Cirugiao Geral\b",
    r"(?i)\bMedico Cirujano Y Partero\b",
    r"(?i)\bGeneral Surgery &Laser Applications\b",
    r"(?i)\bConsultant General Surgery\b",
    r"(?i)\bChirurgie Generale\b",
    r"(?i)\bGeneral Suegery\b",
    r"(?i)\bM S  General Surgery\b",
    r"(?i)\bCirurgia Geral\b",
    r"(?i)\bCirujano General Medico Esteticista\b",
    r"(?i)\bCirujano General\b",
    r"(?i)\bGeneral Surgery Bariatric Cosmetic\b",
    r"(?i)\bGeneral Surgery Specialist& Aestetic Medecin\b",
    r"(?i)\bConsultants General Surgery\b",
]

# Exact matches that should be updated
general_surgeon_exact_matches = {
    "General Surgeon",
    "General Surgeon Specialist",
    "Board Certified General Surgeon",
    "Certified General Surgeon",
    # Spanish form matches
    "Cirujano General",
    "Cirujano Generalista",
    "Cirujano de Atención Primaria",
    # Other possible variations
    "Doctor General Surgeon",
    "Doctor en Cirugía General",
    "General Surgical Specialist",
    "General Surgical Doctor",
    "General Surgical Physician",
    "Licensed General Surgeon",
    "Surgeon",
    "Surgery",
    'Chirurgie',
    'Cirurgia',
    'Chirurgie',
    'Cirurgia',
    'Cirujano',
    'Chirurgie Generale',
    'CirujanoMedico',
    'General Suegery',
    'Cirurgia Geral',
    'Surgeries',
    'Surgeon',
    "surgeon",
    "surgery",
    'chirurgie',
    'cirurgia',
    'chirurgie',
    'cirurgia',
    'cirujano',
    'chirurgie Generale',
    'cirujanoMedico',
    'general suegery',
    'cirurgia geral',
    'surgeries',
    'surgeon',
    'Medico Chirurgo',
    'medico chirurgo',
    'Khirurg',
    'khirurg',
    'Medico Cirujano',
    'medico cirujano',
    'Medico Chirurgo',
    'medico chirurgo',
    'Medico Cirujana',
    'medico cirujana',
    'CirujanoMedico',
    'cirujanomedico',
    'Medico Y Cirujano',
    'medico Y cirujano',
    'Consultant SurgeonClinical Director Im&T',
    'Director Of Surgery Center',
    'Proctologist',
    'Sirgeon',
    'Medico Especialista En Cirugia General Estetica',
    'Bariatric Surgeon',
    'Cirurgiao',
    'Physician Surgeonowner',
    'Surgeonowner',
    'MEDICO CIRUJANO',
    'Medico cirujano',
    'President Surgeon',
    'Surgical Specialist',
}

# Define patterns (these should NOT be changed)
general_surgeon_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Cirurgia Plastica',
    'Cirujano Plastico',
    'Administrative Coordinator General Surgery & Dermatology',
    'Program Manager General Surgery Residency Program',
    'Residency Program General Surgery Coordinator',
    'Site Manager - General Surgery',
    'Trauma & General Surgery App Supervisor',
    'General Surgery Research Resident',
    'Manager Marketing General Surgery & Gyn',
    'General Surgery Inventory Coordinator',
    'Nurse Practitioner - Acute Care General Surgery',
    'Nurse Practitioner General Surgery',
    'Practice Administrator General Surgery',
    'Program Director General Surgery Residency',
    'Group Manager Marketing General Surgery & Gyn',
    'Plastic & General Surgery Coordinator',
    'General Surgery Clinical Coordinator',
    'General Surgery Chief Resident',
    'Graduate Medical Education & General Surgery Program Coordinator',
    'Administrative Assistant General Surgery',
    'General Surgery Residency Interim Program Director',
    'Program Director General Surgery',
    'Physician Assistant - General Surgery',
    'Orthopedic & General Surgery Nurse Case Manager',
    'General Surgery Territory Sales Manager',
    'Surgical Coordinator Colorectal & General Surgery',
    'Client Developer & General Surgery',
    'Pediatric General Surgery Nurse Practitioner',
    'Medical Director Region General Surgery South',
    'Pgy- 2 General Surgery Resident',
}

# Create a mask for General Surgeon
mask_general_surgeon = df['speciality'].str.contains('|'.join(general_surgeon_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(general_surgeon_exact_matches)
mask_general_surgeon_exclusions = df['speciality'].isin(general_surgeon_exclusions)

# Final mask: Select General Surgeon
mask_general_surgeon_final = mask_general_surgeon & ~mask_general_surgeon_exclusions

# Store the original values that will be replaced
original_general_surgeon_values = df.loc[mask_general_surgeon_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_general_surgeon_final, 'speciality'] = 'General Surgeon'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: General Surgeon", 'green'))
print(df.loc[mask_general_surgeon_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_general_surgeon_values = df.loc[mask_general_surgeon_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: General Surgeon", "cyan"))
for original_general_surgeon_value in original_general_surgeon_values:
    print(f"✅ {original_general_surgeon_value} → General Surgeon")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with General Surgeon:", 'red'))
print(grouped_general_surgeon_values)

# Print summary
matched_count_general_surgeon = mask_general_surgeon_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to General Surgeon: "
        f"{matched_count_general_surgeon}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Dental Assistant related titles

dental_assistant_variants = [
    # Standard title variations
    r"(?i)\bDental\s?Assistant\b",
    r"(?i)\bDental\s?Tech\b",
    r"(?i)\bDental\s?Nurse\b",
    r"(?i)\bDental\s?Hygienist\b",
    r"(?i)\bOral\s?Health\s?Assistant\b",
    r"(?i)\bDental\s?Support\s?Staff\b",

    # Common misspellings and case mistakes
    r"(?i)\bDentel\s?Assistant\b",
    r"(?i)\bDantal\s?Assistant\b",
    r"(?i)\bDentel\s?Tech\b",
    r"(?i)\bDantal\s?Nurse\b",

    # Variants in Spanish and other languages
    r"(?i)\bAsistente\s?Dental\b",
    r"(?i)\bAyudante\s?Dental\b",
    r"(?i)\bAuxiliar\s?Dental\b",
    r"(?i)\bTécnico\s?Dental\b",
    r"(?i)\bHigienista\s?Dental\b",

    # Other possible variations
    r"(?i)\bCertified\s?Dental\s?Assistant\b",
    r"(?i)\bLicensed\s?Dental\s?Assistant\b",
    r"(?i)\bRegistered\s?Dental\s?Assistant\b",
    r"(?i)\bDental Assistant\b",
]

# Exact matches that should be updated
dental_assistant_exact_matches = {
    r"(?i)\bDentel\s?Assistant\b",
    r"(?i)\bDantal\s?Assistant\b",
    r"(?i)\bDentel\s?Tech\b",
    r"(?i)\bDantal\s?Nurse\b",
    r"(?i)\bAsistente\s?Dental\b",
    r"(?i)\bAyudante\s?Dental\b",
    r"(?i)\bAuxiliar\s?Dental\b",
    r"(?i)\bTécnico\s?Dental\b",
    r"(?i)\bHigienista\s?Dental\b",
    r"(?i)\bDental Clinic\b",
    r"(?i)\bDental Service Assistant Chief\b",   
}

# Define patterns (these should NOT be changed)
dental_assistant_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident'
}

# Create a mask for Dental Assistant
mask_dental_assistant = df['speciality'].str.contains('|'.join(dental_assistant_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(dental_assistant_exact_matches)
mask_dental_assistant_exclusions = df['speciality'].isin(dental_assistant_exclusions)

# Final mask: Select Dental Assistant
mask_dental_assistant_final = mask_dental_assistant & ~mask_dental_assistant_exclusions

# Store the original values that will be replaced
original_dental_assistant_values = df.loc[mask_dental_assistant_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_dental_assistant_final, 'speciality'] = 'Dental Assistant'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Dental Assistant", 'green'))
print(df.loc[mask_dental_assistant_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_dental_assistant_values = df.loc[mask_dental_assistant_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Dental Assistant", "cyan"))
for original_dental_assistant_value in original_dental_assistant_values:
    print(f"✅ {original_dental_assistant_value} → Dental Assistant")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Dental Assistant:", 'red'))
print(grouped_dental_assistant_values)

# Print summary
matched_count_dental_assistant = mask_dental_assistant_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Dental Assistant: "
        f"{matched_count_dental_assistant}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Hospital Manager related titles

hospital_manager_variants = [
    r"(?i)\bHospital\s?Manager\b",

    # Common misspellings and case errors
    r"(?i)\bHosptial\s?Manager\b",
    r"(?i)\bHospital\s?Manger\b",
    r"(?i)\bHospitl\s?Manager\b",
    r"(?i)\bHospital\s?Mngr\b",
    r"(?i)\bOffice Assistant\b",

    # Spanish variants
    r"(?i)\bGerente\s?de\s?Hospital\b",
    r"(?i)\bAdministrador\s?de\s?Hospital\b",

    # Other possible variations
    r"(?i)\bHealthcare\s?Manager\b",
    r"(?i)\bHospital\s?Administrator\b",
    r"(?i)\bMedical\s?Facility\s?Manager\b",
    r"(?i)\bDirector\s?of\s?Hospital\s?Operations\b",
    r"(?i)\bPublic Hospital\b",
    r"(?i)\bAssistente Direzione Medica\b",
    r"(?i)\bDuty Manager\b",
    r"(?i)\bFacilities Manager\b",
    r"(?i)\bFacility Manager\b",
    r"(?i)\bRnOffice Manager\b",
    r"(?i)\bIcu\b",
    r"(?i)\bInventory Manager\b",
    r"(?i)\bMedical Coordination\b",
    r"(?i)\bService Manager\b",
    r"(?i)\bHospital Provincial De Pemba\b",
    r"(?i)\bService Manager\b",
    r"(?i)\bOperational Also Roustabouts\b",
    r"(?i)\bHealth Management\b",
    r"(?i)\bOperation Room Management\b",
    r"(?i)\bHead Of Hospital\b",
    r"(?i)\bHead Office\b",
    r"(?i)\bReceptionist\b",
    r"(?i)\bAnti Aging Center Manager\b",
    r"(?i)\bAssociate Chief Division Of Hospital Medicine\b",
    r"(?i)\bMedical Officer\b",
    r"(?i)\bCho\b",
    r"(?i)\bOffice ManagerPatient Coordinator\b",
    r"(?i)\bDirector of Aesthetic scheduling\b",
    r"(?i)\bFront Desk Patient Concierge\b",
]

# Exact matches that should be updated
hospital_manager_exact_matches = {
    "Hospital Manager",
    "Hosptial Manager",
    "Hospital Manger",
    "Hospitl Manager",
    "Hospital Mngr",
    # Spanish variants
    "Gerente de Hospital",
    "Administrador de Hospital",
    # Other possible variations
    "Healthcare Manager",
    "Hospital Administrator",
    "Medical Facility Manager",
    "Director of Hospital Operations",
    "Health Business/Administration",
    "Health BusinessAdministration",
    'Planner',
    'planner',
    'Coordinator',
    'coordinator',
    'Quality Officer',
    'Hospital Director',
    'Post-16 Manager',
    'Client Services Director',
    'Director Of Customer Experience',
    'Gerante Centre Esthetique',
    'DirectorRn',
    'Director Of Admissions',
    'Group Coordinator',
    'Fukai Ordho Office',
    'Centre Esthetique',
    'Manager Client Services',
    'Head Of Customer Service',
    'The Key To Generating New Patients',
    'Center Manager',
    'Quality Assurance',
    'General Coordinator',
    'Client Services Manager',
    'Director Of Client Services',
    'RnManager',
    'Hospitaldirector',
    'ID Manager',
    'Cho',
    'Patient Coordinator',
    'Front Desk Manager',
    'Client Services Coordinator',
    'Front Desk',
    'Quality Control Manager',
    'Office Coordinator',
    'Administrative Assistant General Surgery',
    'Administrative Coordinator General Surgery & Dermatology',
    'Chief Operating Officer Physicians Healthcare Collaborative',
    'Chief Physician Administrative Services Officer',
    'Chief Physician Executive',
    'Chief Physician Executive Officer Patient Experience',
    'Chief Physician Services Officer',
    'Epic Resolute Hospital Billing & Physician Billing Senior Analyst',
    'Physicians Coordinator',
    'Physicians Surgery Centers Llc',
}

# Define patterns (these should NOT be changed)
hospital_manager_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Hospital Manager
mask_hospital_manager = df['speciality'].str.contains('|'.join(hospital_manager_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(hospital_manager_exact_matches)
mask_hospital_manager_exclusions = df['speciality'].isin(hospital_manager_exclusions)

# Final mask: Select Hospital Manager
mask_hospital_manager_final = mask_hospital_manager & ~mask_hospital_manager_exclusions

# Store the original values that will be replaced
original_hospital_manager_values = df.loc[mask_hospital_manager_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_hospital_manager_final, 'speciality'] = 'Hospital Manager'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Hospital Manager", 'green'))
print(df.loc[mask_hospital_manager_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_hospital_manager_values = df.loc[mask_hospital_manager_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Hospital Manager", "cyan"))
for original_hospital_manager_value in original_hospital_manager_values:
    print(f"✅ {original_hospital_manager_value} → Hospital Manager")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Hospital Manager:", 'red'))
print(grouped_hospital_manager_values)

# Print summary
matched_count_hospital_manager = mask_hospital_manager_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Hospital Manager: "
        f"{matched_count_hospital_manager}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Anaesthesist related titles
anaesthesist_variants = [
    r"(?i)\bAnaesthesist\b",
    r"(?i)\bAnaesthetist\b",
    r"(?i)\bAnesthesist\b",
    r"(?i)\bAnesthetist\b",
    r"(?i)\bAnasthesist\b",
    r"(?i)\bAnasthitist\b",
    r"(?i)\bAnaesthesiology\b",
    r"(?i)\bAnaesthesia Specialist\b",
    r"(?i)\bAnaesthesia Doctor\b",
    r"(?i)\bAnesthetic Medicine\b",
    r"(?i)Anestesiologist",
    r"(?i)Anesthesia",
    r"(?i)Anesthesiste",
    r"(?i)Anesthesiology",
    r"(?i)Anestesia",
    r"(?i)Anaesthesiologist",
    r"(?i)Aneasthetic",
    r"(?i)Anasthesie",
]

# Exact matches that should be updated
anaesthesist_exact_matches = {
    'Anaesthesist',
    'Anaesthetist',
    'Anesthesist',
    'Anesthetist',
    'Anasthesist',
    'Anasthitist',
    'Anaesthesiology',
    'Anaesthesia Specialist',
    'Anaesthesia Doctor',
    'Anaesthetist Specialist',
    'Anaesthesist Doctor',
    'Anaesthesia Expert',
    'Anesthesiology Specialist',

    # Case-related exclusions
    'anaesthesist',
    'ANAESTHESIST',
    'ANAESTHETIST',
    'anesthesist',
    'ANESTHESIST',
    'AnESTHESIst',
    'anESTHESIST',
    'anAesthesIst',
    'aNAesthesist',

    # Spanish related exclusions
    'Anestesista',
    'Médico Anestesista',
    'Especialista en Anestesia',
    'Anestesista Médico',
    'Especialista en Anestesiología',
    'Médico Anestesiólogo',
    'Especialista en Anestesista',
    'Médico Anestésico',
    'Anestesista General',
    'Anestesiólogo',
    'Anestesista de Salud',
    'Anestesia General',
    'Médico en Anestesia',
    'Especialista en Anestesia General',
    'Anaesthesia',
    'Interventional Pain',
    'Interventional Pain Physician',
    'Pain Management Physician',
    'Pain Physician & Ambulatory Surgical Center Medical Director',
}

# Define patterns for  & Resident & Professor (these should NOT be changed)
anaesthesist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Professor Of Anesthesia',
    'Physician Recruiter - Anesthesiology Family & Internal Medicine Consultant',
}

# Create a mask for Anaesthesist
mask_anaesthesist = df['speciality'].str.contains('|'.join(anaesthesist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(anaesthesist_exact_matches)
mask_anaesthesist_exclusions = df['speciality'].isin(anaesthesist_exclusions)

# Final mask: Select Anaesthesist
mask_anaesthesist_final = mask_anaesthesist & ~mask_anaesthesist_exclusions

# Store the original values that will be replaced
original_anaesthesist_values = df.loc[mask_anaesthesist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_anaesthesist_final, 'speciality'] = 'Anaesthesist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Anaesthesist", 'green'))
print(df.loc[mask_anaesthesist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_anaesthesist_values = df.loc[mask_anaesthesist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Anaesthesist", "cyan"))
for original_anaesthesist_value in original_anaesthesist_values:
    print(f"✅ {original_anaesthesist_value} → Anaesthesist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Anaesthesist:", 'red'))
print(grouped_anaesthesist_values)

# Print summary
matched_count_anaesthesist = mask_anaesthesist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Anaesthesist: "
        f"{matched_count_anaesthesist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Practice Manager related titles

practice_manager_variants = [
    # Standard Titles & Variants
    r"(?i)\bPractice Manager\b",
    r"(?i)\bMedical Practice Manager\b",
    r"(?i)\bHealthcare Practice Manager\b",
    r"(?i)\bPractice Operations Manager\b",
    r"(?i)\bMedical Office Manager\b",
    r"(?i)\bPm\b",
    r"(?i)\bPractice Admin\b",
    r"(?i)\bPractice Administrator\b",
    r"(?i)\bPractice Head\b",
    r"(?i)\bPractice Growth & Profitability\b",
    r"(?i)\bPractice Development Manager\b",
    r"(?i)\bSenior Service Specialist\b",
    r"(?i)\bPractice Development\b",
    r"(?i)\bPractice Coordinator\b",

    # Misspellings & Typographical Errors
    r"(?i)\bPractise Manager\b",
    r"(?i)\bPractice Manger\b",
    r"(?i)\bPractic Manager\b",
    r"(?i)\bPractise Manger\b",
    r"(?i)\bPractioner Manager\b",
    r"(?i)\bPractice Maneger\b",

    # Case Variations
    r"(?i)\bpractice manager\b",
    r"(?i)\bPRACTICE MANAGER\b",
    r"(?i)\bPrAcTiCe MaNaGeR\b",

    # Spanish Variants
    r"(?i)\bGerente de Práctica\b",
    r"(?i)\bGerente de Oficina Médica\b",
    r"(?i)\bGerente de Consultorio Médico\b",
    r"(?i)\bAdministrador de Consultorio Médico\b",

    # Other Possible Variations
    r"(?i)\bHealthcare Administrator\b",
    r"(?i)\bMedical Office Administrator\b",
    r"(?i)\bPractice Operations Coordinator\b",
    r"(?i)\bPractice Operations Director\b",
    r"(?i)\bHealthcare Facility Manager\b",
    r"(?i)\bHealthcare Facility Manager\b",
    r"(?i)\bPratiche\b",
    r"(?i)\bOffice Manager Medical Affairs Administration\b",
    r"(?i)\bPractice Management\b",
]

# Exact matches that should be updated
practice_manager_exact_matches = {
    "Practice Manager",
    "Medical Practice Manager",
    "Healthcare Practice Manager",
    "Practice Operations Manager",
    "Medical Office Manager",
    "Practise Manager",
    "Practice Manger",
    "Practic Manager",
    "Practise Manger",
    "Practioner Manager",
    "Practice Maneger",
    "Gerente de Práctica",
    "Gerente de Oficina Médica",
    "Gerente de Consultorio Médico",
    "Administrador de Consultorio Médico",
    "Healthcare Administrator",
    "Medical Office Administrator",
    "Practice Operations Coordinator",
    "Practice Operations Director",
    "Healthcare Facility Manager",
    'Management',
    'CeoPractice Manager',
    'Aesthetics Program Manager - Physician Advocate',
    'Training Manager',
    'Clinical Training Director',
    'Practice Director',
    'Director Of Aesthetic Training',
    'Foundation Training',
    'OwnerPractice Manager',
    'Pracitce Manager',
    'Practice Makkar',
    'Prcactice Manager',
    'RNPractice Manager',
    'RN Administrator',
    'Practice admin',
    'Practice Administrator',
    'Director Training & Development',
    'Director of Training & Technical Support',
    'Director Of Physician Network Development & Practice Acquisitions',
    'Director Of Physician Practices',
    'Director Physician Practices',
    'Physician Practice Liaison',
    'Program Manager & Senior Acquisitions Physician',
    'Program Manager & Tpmg Health & Wellness Physician',
    'Program Manager Community Relations & Outreach Physician',
    'Program Manager General Surgery Residency Program',
    'Director of Aesthetics & Customer Service',
}

# # Define patterns (these should NOT be changed)
# practice_manager_exclusions = r'\b(?:Plastic)|(?:Physician)\b'

practice_manager_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Practice Manager
mask_practice_manager = df['speciality'].str.contains('|'.join(practice_manager_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(practice_manager_exact_matches)

# mask_practice_manager_exclusions = df['speciality'].str.contains(practice_manager_exclusions, case=False, na=False, regex=True)
mask_practice_manager_exclusions = df['speciality'].isin(practice_manager_exclusions)

# Final mask: Select Practice Manager
mask_practice_manager_final = mask_practice_manager & ~mask_practice_manager_exclusions

# Store the original values that will be replaced
original_practice_manager_values = df.loc[mask_practice_manager_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_practice_manager_final, 'speciality'] = 'Practice Manager'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Practice Manager", 'green'))
print(df.loc[mask_practice_manager_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_practice_manager_values = df.loc[mask_practice_manager_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Practice Manager", "cyan"))
for original_practice_manager_value in original_practice_manager_values:
    print(f"✅ {original_practice_manager_value} → Practice Manager")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Practice Manager:", 'red'))
print(grouped_practice_manager_values)

# Print summary
matched_count_practice_manager = mask_practice_manager_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Practice Manager: "
        f"{matched_count_practice_manager}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Student Resident Fellow related titles

student_variants = [
# Standard Titles & Variants
    r"(?i)\bStudent Resident Fellow\b",  # General term for Student Resident Fellow
    r"(?i)\bMedical Student\b",  # Common term for medical students
    r"(?i)\bResident Physician\b",  # Resident physician
    r"(?i)\bResident Doctor\b",  # Another common term for resident doctors
    r"(?i)\bFellow Physician\b",  # Fellow of a specialty
    r"(?i)\bResident\b",  # Fellow of a specialty
    r"(?i)\bStudent\b",  # Fellow of a specialty
    r"(?i)\bFellow\b",  # Fellow of a specialty
    r"(?i)\bIntern\b",  # Fellow of a specialty
    r"(?i)\bStudy\b",
    r"(?i)\bMedica Residente\b",
    r"(?i)\bMedico Residente\b",
    r"(?i)Residente",
    r"(?i)Estudante",
    r"(?i)\bMedical School\b",
    r"(?i)\bTrainee\b",
    r"(?i)\bStudents\b",
    r"(?i)\bSenior Resident Department Of Plastic Surgery\b",
    r"(?i)\bEstudiante\b",
    r"(?i)\bResidence\b",

    # Misspellings & Typographical Errors
    r"(?i)\bStudnet Resident Fellow\b",  # Common misspelling
    r"(?i)\bSudent Resident Fellow\b",  # Typo with missing 't'
    r"(?i)\bStudent Residen Fellow\b",  # Misspelling with 'Resident' typo
    r"(?i)\bStduent Resident Fellow\b",  # Misspelling with 't' misplaced
    r"(?i)\bStuden Resident Fellow\b",  # Missing 't' in Student

    # Case Variations
    r"(?i)\bSTUDENT RESIDENT FELLOW\b",  # Uppercase variant
    r"(?i)\bstudent resident fellow\b",  # Lowercase variant
    r"(?i)\bStUdEnT ReSiDeNt FeLlOw\b",  # Mixed case variant

    # Spanish Variants
    r"(?i)\bEstudiante Residente\b",  # Student Resident in Spanish
    r"(?i)\bMédico Residente\b",  # Resident Physician in Spanish
    r"(?i)\bResidente Médico\b",  # Another variation for Resident Doctor in Spanish
    r"(?i)\bFellow Médico\b",  # Fellow Physician in Spanish

    # Hybrid Spanish-English Variants
    r"(?i)\bEstudiante Médico Residente\b",  # Hybrid term for Medical Student Resident
    r"(?i)\bFellow Residente Médico\b",  # Hybrid term for Fellow Resident Physician

    # Other Possible Variations (Doctor Forms, Specialist Forms)
    r"(?i)\bResident Physician Fellow\b",  # Specialized term for residency fellow
    r"(?i)\bMedical Resident Fellow\b",  # Common term for medical resident fellows
    r"(?i)\bDoctoral Fellow\b",  # Doctoral-level fellow
    r"(?i)\bSpecialty Fellow\b",  # Specialty-focused fellowship term
    r"(?i)\bDoctor Fellow\b",  # Variant for a general Doctor Fellow
]

# Exact matches that should be updated
student_exact_matches = {
    'Student/Resident',
    'StudentResident',
    'Student Resident',
    'Junior Doctor',
    'Estudannte',
    'Etudiant',
    'ResidentPeriodontist',
    'Estudande de Biomedicina',
}

# # Define patterns (these should NOT be changed)
# student_exclusions = r'\b(?:Plastic)|(?:Physician)\b'

student_exclusions = {'Nurse'}

# Create a mask for Student Resident Fellow
mask_student = df['speciality'].str.contains('|'.join(student_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(student_exact_matches)

# mask_student_exclusions = df['speciality'].str.contains(student_exclusions, case=False, na=False, regex=True)
mask_student_exclusions = df['speciality'].isin(student_exclusions)

# Final mask: Select Student Resident Fellow
mask_student_final = mask_student & ~mask_student_exclusions

# Store the original values that will be replaced
original_student_values = df.loc[mask_student_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_student_final, 'speciality'] = 'Student Resident Fellow'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Student Resident Fellow", 'green'))
print(df.loc[mask_student_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_student_values = df.loc[mask_student_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Student Resident Fellow", "cyan"))
for original_student_value in original_student_values:
    print(f"✅ {original_student_value} → Student Resident Fellow")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Student Resident Fellow:", 'red'))
print(grouped_student_values)

# Print summary
matched_count_student = mask_student_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Student Resident Fellow: "
        f"{matched_count_student}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Oncologist related titles

oncologist_variants = [
    # Standard Titles & Abbreviations
    r"(?i)\bOncologist\b",
    r"(?i)\bMedical\s?Oncologist\b",
    r"(?i)\bClinical\s?Oncologist\b",
    r"(?i)\bRadiation\s?Oncologist\b",
    r"(?i)\bSurgical\s?Oncologist\b",
    r"(?i)\bPediatric\s?Oncologist\b",
    r"(?i)\bGynecologic\s?Oncologist\b",
    r"(?i)\bNeuro\s?Oncologist\b",
    r"(?i)\bBreast\s?Oncologist\b",
    r"(?i)\bThoracic\s?Oncologist\b",
    r"(?i)\bGenitourinary\s?Oncologist\b",
    r"(?i)\bGastrointestinal\s?Oncologist\b",

    # Misspellings & Typographical Errors
    r"(?i)\bOncolgist\b",
    r"(?i)\bOncolojist\b",
    r"(?i)\bOnclogist\b",
    r"(?i)\bOncologest\b",
    r"(?i)\bOncologis\b",
    r"(?i)\bOncologits\b",
    r"(?i)\bOnckologist\b",
    r"(?i)\bOnkologist\b",

    # Case Variations
    r"(?i)\boncologist\b",
    r"(?i)\bOncologist\b",
    r"(?i)\bONCOLOGIST\b",
    r"(?i)\bOncoLogist\b",

    # Spanish Variants
    r"(?i)\bOncólogo\b",
    r"(?i)\bMédico\s?Oncólogo\b",
    r"(?i)\bEspecialista\s?en\s?Oncología\b",
    r"(?i)\bOncólogo\s?Médico\b",
    r"(?i)\bOncólogo\s?Clínico\b",
    r"(?i)\bOncólogo\s?Radioterápico\b",
    r"(?i)\bOncólogo\s?Pediátrico\b",
    r"(?i)\bCirujano\s?Oncólogo\b",

    # Other Possible Variations (Including Doctor/Specialist Titles)
    r"(?i)\bOncology\s?Specialist\b",
    r"(?i)\bCancer\s?Specialist\b",
    r"(?i)\bTumor\s?Specialist\b",
    r"(?i)\bOncological\s?Surgeon\b",
    r"(?i)\bOncological\s?Physician\b",
    r"(?i)\bOnco\b",
    r"(?i)\bChief Pediatric Oncology\b",
]

# Exact matches that should be updated
oncologist_exact_matches = {
    "Oncologist",
    "Medical Oncologist",
    "Clinical Oncologist",
    "Radiation Oncologist",
    "Surgical Oncologist",
    "Pediatric Oncologist",
    "Gynecologic Oncologist",
    "Neuro Oncologist",
    "Breast Oncologist",
    "Thoracic Oncologist",
    "Genitourinary Oncologist",
    "Gastrointestinal Oncologist",
    "Oncolgist",
    "Oncolojist",
    "Onclogist",
    "Oncologest",
    "Oncologis",
    "Oncologits",
    "Onckologist",
    "Onkologist",
    "Oncólogo",
    "Médico Oncólogo",
    "Especialista en Oncología",
    "Oncólogo Médico",
    "Oncólogo Clínico",
    "Oncólogo Radioterápico",
    "Oncólogo Pediátrico",
    "Cirujano Oncólogo",
    "Oncology Specialist",
    "Cancer Specialist",
    "Tumor Specialist",
    "Oncological Surgeon",
    "Oncological Physician",
}

# # Define patterns (these should NOT be changed)
# oncologist_exclusions = r'\b(?:Plastic)|(?:Physician)\b'

oncologist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Resident Physician - Radiation Oncologist',
}

# Create a mask for Oncologist
mask_oncologist = df['speciality'].str.contains('|'.join(oncologist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(oncologist_exact_matches)

# mask_oncologist_exclusions = df['speciality'].str.contains(oncologist_exclusions, case=False, na=False, regex=True)
mask_oncologist_exclusions = df['speciality'].isin(oncologist_exclusions)

# Final mask: Select Oncologist
mask_oncologist_final = mask_oncologist & ~mask_oncologist_exclusions

# Store the original values that will be replaced
original_oncologist_values = df.loc[mask_oncologist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_oncologist_final, 'speciality'] = 'Oncologist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Oncologist", 'green'))
print(df.loc[mask_oncologist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_oncologist_values = df.loc[mask_oncologist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Oncologist", "cyan"))
for original_oncologist_value in original_oncologist_values:
    print(f"✅ {original_oncologist_value} → Oncologist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Oncologist:", 'red'))
print(grouped_oncologist_values)

# Print summary
matched_count_oncologist = mask_oncologist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Oncologist: "
        f"{matched_count_oncologist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for ENT Ear Nose Throat Physician related titles

ENT_variants = [
    r"(?i)\bENT\s?physician\b",
    r"(?i)\bENT\s?doctor\b",
    r"(?i)\bENT\s?specialist\b",
    r"(?i)\bENT\s?specialist\s?doctor\b",
    r"(?i)\bENT\s?consultant\b",
    r"(?i)\bENT\s?practitioner\b",
    r"(?i)\bENT\s?expert\b",
    r"(?i)\bENT\s?surgeon\b",
    r"(?i)\bENT\s?surgeon\s?doctor\b",
    r"(?i)\bENT\s?MD\b",
    r"(?i)\bENT\s?physician\s?MD\b",
    r"(?i)\bENT\s?physician\s?specialist\b",
    r"(?i)\bboard\s?certified\s?ENT\s?physician\b",
    # Spanish variants
    r"(?i)\botorrihinolaringólogo\b",
    r"(?i)\botorrihinolaringóloga\b",
    r"(?i)\botorrihinolaringólogos\b",
    r"(?i)\botorrihinolaringólogas\b",
    r"(?i)\bespecialista\s?en\s?otorinolaringología\b",
    r"(?i)\bdoctor\s?en\s?otorinolaringología\b",
    # Ear, Nose, Throat variations
    r"(?i)\bEar\s?Nose\s?Throat\s?specialist\b",
    r"(?i)\bEar\s?Nose\s?Throat\s?doctor\b",
    r"(?i)\bEar\s?Nose\s?Throat\s?physician\b",
    r"(?i)\bEar\s?Nose\s?Throat\s?surgeon\b",
    r"(?i)\bEar\s?Nose\s?Throat\s?specialist\s?doctor\b",
    r"(?i)\bEar\s?Nose\s?Throat\s?MD\b",
    r"(?i)\bENT\s?Ear\s?Nose\s?Throat\b",
    # Misspellings
    r"(?i)\bOTR\s?physician\b",
    r"(?i)\bENTP\b",
    r"(?i)\bENT\s?phyician\b",
    r"(?i)\bENT\s?phyisician\b",
    r"(?i)\bENT\s?dr\b",
    # Case-related errors
    r"(?i)\bENT\s?PHYSICIAN\b",
    r"(?i)\bENT\s?Physician\b",
    r"(?i)\bENT\s?physician\b",
    # Other Possible Variations
    r"(?i)\bboard\s?certified\s?ENT\s?physician\b",
    r"(?i)\bENT\s?consultant\b",
    r"(?i)\bENT\s?expert\b",
    r"(?i)\bENT\s?specialist\b",
    r"(?i)\bENT\s?doctor\b",
    r"(?i)\bENT\s?physician\s?specialist\b",
    r"(?i)\bMedica Otorrinolaringologista\b",
    r"(?i)\bOtorrinolaringologista\b",
    r"(?i)\bOtorrinolaringologista\b",
    r"(?i)\bOtolayngologist\b",
    r"(?i)\bOtorhinolaryngology\b",
    r"(?i)Otorrino Cirugia Estetica",
    r"(?i)\bOtolarygology\b",
    r"(?i)\bOtorhinolaryngology\b",
]

# Exact matches that should be updated
ENT_exact_matches = {
    'ENT Ear Nose & Throat Physician',
    "ENT Physician",
    "ENT Doctor",
    "ENT Specialist",
    "ENT Specialist Doctor",
    "ENT Consultant",
    "ENT Practitioner",
    "ENT Expert",
    "ENT Surgeon",
    "ENT Surgeon Doctor",
    "ENT MD",
    "ENT Physician MD",
    "ENT Physician Specialist",
    "Board Certified ENT Ear Nose Throat Physician",
    "Otorrihinolaringólogo",
    "Otorrihinolaringóloga",
    "Otorrihinolaringólogos",
    "Otorrihinolaringólogas",
    "Especialista en Otorrinolaringología",
    "Doctor en Otorrinolaringología",
    "OTR Physician",
    "ENTP",
    "ENT Phyician",
    "ENT Phyisician",
    "ENT Dr",
    "ENT PHYSICIAN",
    "ent physician",
    "Ear Nose Throat Specialist",
    "Ear Nose Throat Doctor",
    "Ear Nose Throat Physician",
    "Ear Nose Throat Surgeon",
    "Ear Nose Throat Specialist Doctor",
    "Ear Nose Throat MD",
    "ENT Ear Nose Throat",
    "Ent Ear Nose & Throat",
    'Otolarygology',
    'Otorhinolaryngology',
    "Otorrinolaringologista",
    "Otorrinolaringologista",
    "Otolayngologist",
    "Otolaryngology",
    "Otorhinolaryngology",
    'Ent',
    'Doctor Ent',
}

# Define patterns (these should NOT be changed)
ENT_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Ent Resident',
    'Professor Of Otorhinolaryngology',
    'Ent Plastic Surgery',
    'Consultant Ent Rhinologist & Facial Plastic Surgeon',
    'University ProfessorOtolaryngologyFacial Plastic',
    "ENT Head and Neck Physician",
    "ENT Head and Neck Doctor",
    "ENT Head and Neck Specialist",
    "ENT Head and Neck Surgeon",
    "ENT Head and Neck MD",
    "ENT Head and Neck Consultant",
    "ENT Head and Neck Practitioner",
    "ENT Head and Neck Expert",
    "ENT Head and Neck Surgeon Doctor",
    "Board Certified ENT Head and Neck Physician",
    "Head and Neck ENT Specialist",
    "Head and Neck ENT Doctor",
    "Head and Neck ENT Surgeon",
    "Head and Neck ENT Physician",
    "Head and Neck ENT Consultant",
    "ENT Head & Neck Specialist",
    "ENT Head & Neck Doctor",
    "ENT Head & Neck Surgeon",
    "ENT Head & Neck MD",
    "ENT Head & Neck Consultant",
    "Especialista en Cabeza y Cuello",
    "Cirujano de Cabeza y Cuello",
    "Médico de Cabeza y Cuello",
    "Otorrinolaringólogo de Cabeza y Cuello",
    "Ear Nose Throat Head and Neck Specialist",
    "Ear Nose Throat Head and Neck Doctor",
    "Ear Nose Throat Head and Neck Surgeon",
    "Ear Nose Throat Head and Neck Physician",
    "Ear Nose Throat Head and Neck MD",
    "ENT Ear Nose Throat Head and Neck",
    "ENT H&N Physician",
    "ENT H&N Doctor",
    "ENT H&N Specialist",
    "ENT H&N Surgeon",
    "ENT H&N MD",
    "ENT H&N Consultant",
    'ENT/Head & Neck Specialist',
    'ENTHead & Neck Specialist',
    "EntHead & Neck Surgery",
    "Head & Neck Surgery",
    "EntHead & Neck Surgeon",
    "Head & Neck Surgeon",
    "Head & Neck Specialist",
}

# Create a mask for ENT Ear Nose Throat Physician
mask_ENT = df['speciality'].str.contains('|'.join(ENT_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(ENT_exact_matches)
mask_ENT_exclusions = df['speciality'].isin(ENT_exclusions)

# Final mask: Select ENT Ear Nose Throat Physician
mask_ENT_final = mask_ENT & ~mask_ENT_exclusions

# Store the original values that will be replaced
original_ENT_values = df.loc[mask_ENT_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_ENT_final, 'speciality'] = 'ENT Ear Nose Throat Physician'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: ENT Ear Nose Throat Physician", 'green'))
print(df.loc[mask_ENT_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_ENT_values = df.loc[mask_ENT_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: ENT Ear Nose Throat Physician", "cyan"))
for original_ENT_value in original_ENT_values:
    print(f"✅ {original_ENT_value} → ENT Ear Nose Throat Physician")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with ENT Ear Nose Throat Physician:", 'red'))
print(grouped_ENT_values)

# Print summary
matched_count_ENT = mask_ENT_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to ENT Ear Nose Throat Physician: "
        f"{matched_count_ENT}",
        'red'))

# Fix / Handle ENT Head and Neck which is important value for EUROGIN

ENT_head_neck_variants = [
    r"(?i)\bENT\s?head\s?and\s?neck\s?physician\b",
    r"(?i)\bENT\s?head\s?and\s?neck\s?doctor\b",
    r"(?i)\bENT\s?head\s?and\s?neck\s?specialist\b",
    r"(?i)\bENT\s?head\s?and\s?neck\s?surgeon\b",
    r"(?i)\bENT\s?head\s?and\s?neck\s?MD\b",
    r"(?i)\bENT\s?head\s?and\s?neck\s?consultant\b",
    r"(?i)\bENT\s?head\s?and\s?neck\s?practitioner\b",
    r"(?i)\bENT\s?head\s?and\s?neck\s?expert\b",
    r"(?i)\bENT\s?head\s?and\s?neck\s?surgeon\s?doctor\b",
    r"(?i)\bboard\s?certified\s?ENT\s?head\s?and\s?neck\s?physician\b",
    r"(?i)\bhead\s?and\s?neck\s?ENT\s?specialist\b",
    r"(?i)\bhead\s?and\s?neck\s?ENT\s?doctor\b",
    r"(?i)\bhead\s?and\s?neck\s?ENT\s?surgeon\b",
    r"(?i)\bhead\s?and\s?neck\s?ENT\s?physician\b",
    r"(?i)\bhead\s?and\s?neck\s?ENT\s?consultant\b",
    r"(?i)\bENT\s?head\s?&\s?neck\s?specialist\b",
    r"(?i)\bENT\s?head\s?&\s?neck\s?doctor\b",
    r"(?i)\bENT\s?head\s?&\s?neck\s?surgeon\b",
    r"(?i)\bENT\s?head\s?&\s?neck\s?MD\b",
    r"(?i)\bENT\s?head\s?&\s?neck\s?consultant\b",
    # Spanish Variants
    r"(?i)\bEspecialista\s?en\s?cabeza\s?y\s?cuello\b",
    r"(?i)\bCirujano\s?de\s?cabeza\s?y\s?cuello\b",
    r"(?i)\bMédico\s?de\s?cabeza\s?y\s?cuello\b",
    r"(?i)\bOtorrinolaringólogo\s?de\s?cabeza\s?y\s?cuello\b",
    # Ear, Nose, Throat, Head and Neck Variations
    r"(?i)\bHead\s?and\s?Neck\s?specialist\b",
    r"(?i)\bHead\s?and\s?Neck\s?doctor\b",
    r"(?i)\bHead\s?and\s?Neck\s?surgeon\b",
    r"(?i)\bHead\s?and\s?Neck\s?physician\b",
    r"(?i)\bHead\s?and\s?Neck\s?MD\b",
    # Common Misspellings
    r"(?i)\bENT\s?head\s?and\s?neck\s?phyisician\b",
    r"(?i)\bENT\s?head\s?and\s?neck\s?phyician\b",
    r"(?i)\bENT\s?H&N\s?physician\b",
    r"(?i)\bENT\s?H&N\s?doctor\b",
    r"(?i)\bENT\s?H&N\s?specialist\b",
    r"(?i)\bENT\s?H&N\s?surgeon\b",
    r"(?i)\bENT\s?H&N\s?MD\b",
    r"(?i)\bENT\s?H&N\s?consultant\b",
    r"(?i)\bENT/Head & Neck Specialist\b",
    r"(?i)\bEntHead & Neck Specialist\b",
    r"(?i)\bEntHead & Neck Surgery\b",
    r"(?i)\bHead & Neck Surgery\b",
    r"(?i)\bEntHead & Neck Surgeon\b",
    r"(?i)\bHead & Neck Surgeon\b",
    r"(?i)\bHead & Neck Specialist\b",
    r"(?i)\bENT/Head & Neck Specialist\b"
]


ENT_head_neck_exact_matches = {
    'ENT/Head & Neck Specialist',
    "ENT Head and Neck Physician",
    "ENT Head and Neck Doctor",
    "ENT Head and Neck Specialist",
    "ENT Head and Neck Surgeon",
    "ENT Head and Neck MD",
    "ENT Head and Neck Consultant",
    "ENT Head and Neck Practitioner",
    "ENT Head and Neck Expert",
    "ENT Head and Neck Surgeon Doctor",
    "Board Certified ENT Head and Neck Physician",
    "Head and Neck ENT Specialist",
    "Head and Neck ENT Doctor",
    "Head and Neck ENT Surgeon",
    "Head and Neck ENT Physician",
    "Head and Neck ENT Consultant",
    "ENT Head & Neck Specialist",
    "ENT Head & Neck Doctor",
    "ENT Head & Neck Surgeon",
    "ENT Head & Neck MD",
    "ENT Head & Neck Consultant",
    "Especialista en Cabeza y Cuello",
    "Cirujano de Cabeza y Cuello",
    "Médico de Cabeza y Cuello",
    "Otorrinolaringólogo de Cabeza y Cuello",
    "Ear Nose Throat Head and Neck Specialist",
    "Ear Nose Throat Head and Neck Doctor",
    "Ear Nose Throat Head and Neck Surgeon",
    "Ear Nose Throat Head and Neck Physician",
    "Ear Nose Throat Head and Neck MD",
    "ENT Ear Nose Throat Head and Neck",
    "ENT H&N Physician",
    "ENT H&N Doctor",
    "ENT H&N Specialist",
    "ENT H&N Surgeon",
    "ENT H&N MD",
    "ENT H&N Consultant",
    'ENT/Head & Neck Specialist',
    'ENTHead & Neck Specialist',
    "EntHead & Neck Surgery",
    "Head & Neck Surgery",
    "EntHead & Neck Surgeon",
    "Head & Neck Surgeon",
    "Head & Neck Specialist",
    'Dr  Ent&Hns',
}

# Define patterns (these should NOT be changed)
ENT_head_neck_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Ent Resident',
    'Professor Of Otorhinolaryngology',
    'Ent Plastic Surgery',
    'Consultant Ent Rhinologist & Facial Plastic Surgeon',
    'University ProfessorOtolaryngologyFacial Plastic',
    'ENT Ear Nose Throat Physician',
    "ENT Physician",
    "ENT Doctor",
    "ENT Specialist",
    "ENT Specialist Doctor",
    "ENT Consultant",
    "ENT Practitioner",
    "ENT Expert",
    "ENT Surgeon",
    "ENT Surgeon Doctor",
    "ENT MD",
    "ENT Physician MD",
    "ENT Physician Specialist",
    "Board Certified ENT Ear Nose Throat Physician",
    "Otorrihinolaringólogo",
    "Otorrihinolaringóloga",
    "Otorrihinolaringólogos",
    "Otorrihinolaringólogas",
    "Especialista en Otorrinolaringología",
    "Doctor en Otorrinolaringología",
    "OTR Physician",
    "ENTP",
    "ENT Phyician",
    "ENT Phyisician",
    "ENT Dr",
    "ENT PHYSICIAN",
    "ent physician",
    "Ear Nose Throat Specialist",
    "Ear Nose Throat Doctor",
    "Ear Nose Throat Physician",
    "Ear Nose Throat Surgeon",
    "Ear Nose Throat Specialist Doctor",
    "Ear Nose Throat MD",
    "ENT Ear Nose Throat",
    "Ent Ear Nose & Throat",
}


# Create a mask for ENT Ear Nose Throat Physician
mask_ENT_head_neck = df['speciality'].str.contains('|'.join(ENT_head_neck_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(ENT_head_neck_exact_matches)
mask_ENT_head_neck_exclusions = df['speciality'].isin(ENT_head_neck_exclusions)

# Final mask: Select ENT Ear Nose Throat Physician
mask_ENT_head_neck_final = mask_ENT_head_neck & ~mask_ENT_head_neck_exclusions

# Store the original values that will be replaced
original_ENT_head_neck_values = df.loc[mask_ENT_head_neck_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_ENT_head_neck_final, 'speciality'] = 'ENT Head and Neck Specialist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: ENT Head and Neck Specialist", 'green'))
print(df.loc[mask_ENT_head_neck_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_ENT_head_neck_values = df.loc[mask_ENT_head_neck_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: ENT Head and Neck Specialist", "cyan"))
for original_ENT_head_neck_value in original_ENT_head_neck_values:
    print(f"✅ {original_ENT_head_neck_value} → ENT Head and Neck Specialist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with ENT Head and Neck Specialist:", 'red'))
print(grouped_ENT_head_neck_values)

# Print summary
matched_count_ENT_head_neck = mask_ENT_head_neck_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to ENT Head and Neck Specialist: "
        f"{matched_count_ENT_head_neck}",
        'red'))